@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Iterator.scala 9554 2007-01-04 16:30:16Z odersky $
+// $Id: Iterator.scala 9980 2007-02-13 22:06:58Z odersky $
 
 
 package scala
@@ -14,13 +14,14 @@ package scala
 
 import Predef._
 import collection.mutable.{Buffer, ArrayBuffer}
+import compat.StringBuilder
 
 /** The <code>Iterator</code> object provides various functions for
  *  creating specialized iterators.
  *
  *  @author  Martin Odersky
  *  @author  Matthias Zenger
- *  @version 1.1, 04/02/2004
+ *  @version 1.2, 10/02/2007
  */
 object Iterator {
 
@@ -290,14 +291,15 @@ trait Iterator[+A] {
   private def predicatedIterator(p: A => boolean, isFilter: boolean) = new BufferedIterator[A] {
     private var hd: A = _
     private var ahead: Boolean = false
-    private var hasMore = Iterator.this.hasNext
     private def skip: Unit =
-      while (!ahead && hasMore) {
+      while (!ahead && Iterator.this.hasNext) {
         hd = Iterator.this.next
-        hasMore = Iterator.this.hasNext
-        ahead = p(hd)
+        ahead = !isFilter || p(hd)
       }
-    def hasNext: Boolean = { skip; ahead || isFilter && hasMore }
+    def hasNext: Boolean = { 
+      skip
+      ahead && p(hd)
+    }
     def next: A =
       if (hasNext) { ahead = false; hd }
       else throw new NoSuchElementException("next on empty iterator")
@@ -605,4 +607,42 @@ trait Iterator[+A] {
     }
     res.toList
   }
+
+  /** Returns a string representation of the elements in this iterator. The resulting string
+   *  begins with the string <code>start</code> and is finished by the string
+   *  <code>end</code>. Inside, the string representations of elements (w.r.t.
+   *  the method <code>toString()</code>) are separated by the string
+   *  <code>sep</code>.
+   *  <p/>
+   *  Ex: <br/>
+   *  <code>List(1, 2, 3).mkString("(", "; ", ")") = "(1; 2; 3)"</code>
+   *
+   *  @param start starting string.
+   *  @param sep separator string.
+   *  @param end ending string.
+   *  @return a string representation of this iterable object.
+   */
+  def mkString(start: String, sep: String, end: String): String = {
+    val buf = new StringBuilder()
+    addString(buf, start, sep, end).toString
+  }
+  
+  /** Returns a string representation of this iterable object. The string
+   *  representations of elements (w.r.t. the method <code>toString()</code>)
+   *  are separated by the string <code>sep</code>.
+   *
+   *  @param sep separator string.
+   *  @return a string representation of this iterable object. */
+  def mkString(sep: String): String = this.mkString("", sep, "")
+
+  /** Write all elements of this string into given string builder */
+  def addString(buf: StringBuilder, start: String, sep: String, end: String): StringBuilder = {
+    buf.append(start)
+    val elems = this
+    if (elems.hasNext) buf.append(elems.next)
+    while (elems.hasNext) {
+      buf.append(sep); buf.append(elems.next)
+    }
+    buf.append(end)
+  }
 }
