@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Actor.scala 9681 2007-01-23 14:12:33Z phaller $
+// $Id: Actor.scala 10032 2007-02-19 10:45:23Z phaller $
 
 package scala.actors
 
@@ -19,7 +19,7 @@ import compat.Platform
  * <code>receive</code>, <code>react</code>, <code>reply</code>,
  * etc.
  *
- * @version 0.9.2
+ * @version 0.9.4
  * @author Philipp Haller
  */
 object Actor {
@@ -51,6 +51,19 @@ object Actor {
     a*/
   }
 
+  /**
+   * <p>This function is used for the definition of actors.</p>
+   * <p>The following example demonstrates its usage:</p><pre>
+   * import scala.actors.Actor._
+   * ...
+   * val a = actor {
+   *   ...
+   * }
+   * </pre>
+   *
+   * @param  body  the code block to be executed by the newly created actor
+   * @return       the newly created actor. Note that it is automatically started.
+   */
   def actor(body: => Unit): Actor = synchronized {
     val actor = new Actor {
       def act() = body
@@ -75,6 +88,10 @@ object Actor {
   }
 */
 
+  /**
+   * Receives the next message from the mailbox of the current actor
+   * <code>self</code>.
+   */
   def ? : Any = self.?
 
   /**
@@ -82,8 +99,8 @@ object Actor {
    * <code>self</code>. Blocks if no message matching any of the
    * cases of <code>f</code> can be received.
    *
-   * @param f ...
-   * @return  ...
+   * @param  f a partial function specifying patterns and actions
+   * @return   the result of processing the received message
    */
   def receive[a](f: PartialFunction[Any, a]): a =
     self.receive(f)
@@ -96,36 +113,36 @@ object Actor {
    * received the <code>TIMEOUT</code> action is executed if
    * specified.
    *
-   * @param msec ...
-   * @param f    ...
-   * @return     ...
+   * @param  msec the time span before timeout
+   * @param  f    a partial function specifying patterns and actions
+   * @return      the result of processing the received message
    */
   def receiveWithin[R](msec: long)(f: PartialFunction[Any, R]): R =
     self.receiveWithin(msec)(f)
 
   /**
-   * <code>receive</code> for event-based reactors.
+   * Lightweight variant of <code>receive</code>.
    *
    * Actions in <code>f</code> have to contain the rest of the
    * computation of <code>self</code>, as this method will never
    * return.
    *
-   * @param f ...
-   * @return  ...
+   * @param  f a partial function specifying patterns and actions
+   * @return   this function never returns
    */
   def react(f: PartialFunction[Any, Unit]): Nothing =
     self.react(f)
 
   /**
-   * <code>receiveWithin</code> for event-based reactors.
+   * Lightweight variant of <code>receiveWithin</code>.
    *
    * Actions in <code>f</code> have to contain the rest of the
    * computation of <code>self</code>, as this method will never
    * return.
    *
-   * @param msec ...
-   * @param f    ...
-   * @return     ...
+   * @param  msec the time span before timeout
+   * @param  f    a partial function specifying patterns and actions
+   * @return      this function never returns
    */
   def reactWithin(msec: long)(f: PartialFunction[Any, Unit]): Nothing =
     self.reactWithin(msec)(f)
@@ -172,35 +189,30 @@ object Actor {
    * Causes <code>self</code> to repeatedly execute
    * <code>body</code>.
    *
-   * @param body ...
+   * @param body the code block to be executed
    */
-  def loop(body: => Unit): Unit = {
-    val s = self
-    s.kill = () => { body; s.kill() }
-    body
-    exit("normal")
-  }
+  def loop(body: => Unit): Nothing = body andThen loop(body)
 
   /**
    * Causes <code>self</code> to execute <code>first</code>
    * followed by <code>next</code>.
    *
-   * @param first ...
-   * @param next  ...
+   * @param first the first code block to be executed
+   * @param next  the second code block to be executed
    */
   def seq[a, b](first: => a, next: => b): Nothing = {
     val s = self
     val killNext = s.kill
-    s.kill = () => { s.kill = killNext; next; s.kill() }
+    s.kill = () => { s.kill = killNext; next; exit('normal) }
     first
-    exit("normal")
+    exit('normal)
   }
 
   /**
    * Links <code>self</code> to actor <code>to</code>.
    *
-   * @param to ...
-   * @return   ...
+   * @param  to the actor to link to
+   * @return 
    */
   def link(to: Actor): Actor = self.link(to)
 
@@ -215,7 +227,7 @@ object Actor {
   /**
    * Unlinks <code>self</code> from actor <code>from</code>.
    *
-   * @param from ...
+   * @param from the actor to unlink from
    */
   def unlink(from: Actor): Unit = self.unlink(from)
 
@@ -227,16 +239,16 @@ object Actor {
    * <p>
    *   For each linked actor <code>a</code> with
    *   <code>trapExit</code> set to <code>true</code>, send message
-   *   <code>Exit(self, reason)</code> to <code>a</code>.
+   *   <code>{'EXIT, self, reason}</code> to <code>a</code>.
    * </p>
    * <p>
    *   For each linked actor <code>a</code> with
    *   <code>trapExit</code> set to <code>false</code> (default),
    *   call <code>a.exit(reason)</code> if
-   *   <code>!reason.equals("normal")</code>.
+   *   <code>reason != 'normal</code>.
    * </p>
    */
-  def exit(reason: String): Nothing = self.exit(reason)
+  def exit(reason: AnyRef): Nothing = self.exit(reason)
 }
 
 /**
@@ -245,12 +257,15 @@ object Actor {
  *   implementation of event-based actors.
  * </p>
  * <p>
- *   The main ideas of our approach are explained in the paper<br>
+ *   The main ideas of our approach are explained in the papers<br>
  *   <b>Event-Based Programming without Inversion of Control</b>,
- *   Philipp Haller, Martin Odersky <i>Proc. JMLC 2006</i>
+ *   Philipp Haller and Martin Odersky, <i>Proc. JMLC 2006</i>
+ *   <br><br>
+ *   <b>Actors that Unify Threads and Events</b>,
+ *   Philipp Haller and Martin Odersky, <i>LAMP-REPORT-2007-001, EPFL</i>
  * </p>
  *
- * @version 0.9.2
+ * @version 0.9.4
  * @author Philipp Haller
  */
 trait Actor extends OutputChannel[Any] {
@@ -316,20 +331,28 @@ trait Actor extends OutputChannel[Any] {
       tick()
       val qel = mailbox.extractFirst((m: Any) => f.isDefinedAt(m))
       if (null eq qel) {
-        waitingFor = f.isDefinedAt
-        isSuspended = true
-        received = None
-        suspendActorFor(msec)
-        if (received.isEmpty) {
-          if (f.isDefinedAt(TIMEOUT)) {
-            waitingFor = waitingForNone
-            isSuspended = false
-            val result = f(TIMEOUT)
-            return result
-          }
+        if (msec == 0) {
+          if (f.isDefinedAt(TIMEOUT))
+            return f(TIMEOUT)
           else
             error("unhandled timeout")
         }
+        else {
+          waitingFor = f.isDefinedAt
+          isSuspended = true
+          received = None
+          suspendActorFor(msec)
+          if (received.isEmpty) {
+            if (f.isDefinedAt(TIMEOUT)) {
+              waitingFor = waitingForNone
+              isSuspended = false
+              val result = f(TIMEOUT)
+              return result
+            }
+            else
+              error("unhandled timeout")
+          }
+        }
       } else {
         received = Some(qel.msg)
         sessions = qel.session :: sessions
@@ -395,9 +418,9 @@ trait Actor extends OutputChannel[Any] {
   /**
    * Sends <code>msg</code> to this actor (asynchronous).
    */
-  def !(msg: Any): Unit = send(msg, Actor.self.reply)
+  def !(msg: Any): Unit = send(msg, Actor.self.getReplyChannel)
 
-  def forward(msg: Any): Unit = send(msg, Actor.sender.reply)
+  def forward(msg: Any): Unit = send(msg, Actor.sender.getReplyChannel)
 
   /**
    * Sends <code>msg</code> to this actor and awaits reply
@@ -420,10 +443,48 @@ trait Actor extends OutputChannel[Any] {
     }
   }
 
+  def !!(msg: Any): Future[Any] = {
+    val ftch = new Channel[Any](Actor.self)
+    send(msg, ftch)
+    new Future[Any](ftch) {
+      def apply() =
+        if (isSet) value.get
+        else ch.receive {
+          case any => value = Some(any); any
+        }
+      def isSet = value match {
+        case None => ch.receiveWithin(0) {
+          case TIMEOUT => false
+          case any => value = Some(any); true
+        }
+        case Some(_) => true
+      }
+    }
+  }
+
+  def !![a](msg: Any, f: PartialFunction[Any, a]): Future[a] = {
+    val ftch = new Channel[Any](Actor.self)
+    send(msg, ftch)
+    new Future[a](ftch) {
+      def apply() =
+        if (isSet) value.get
+        else ch.receive {
+          case any => value = Some(f(any)); value.get
+        }
+      def isSet = value match {
+        case None => ch.receiveWithin(0) {
+          case TIMEOUT => false
+          case any => value = Some(f(any)); true
+        }
+        case Some(_) => true
+      }
+    }
+  }
+
   def reply(msg: Any): Unit = session ! msg
 
-  private var rc = new Channel[Any]
-  def reply = rc
+  private var rc = new Channel[Any](this)
+  def getReplyChannel = rc
   def freshReply() = { rc = new Channel[Any]; rc }
 
   def ? : Any = receive {
@@ -438,7 +499,6 @@ trait Actor extends OutputChannel[Any] {
     if (sessions.isEmpty) null
     else sessions.head.asInstanceOf[Channel[Any]]
 
-
   private[actors] var continuation: PartialFunction[Any, Unit] = null
   private[actors] var timeoutPending = false
   private[actors] var isDetached = false
@@ -460,8 +520,6 @@ trait Actor extends OutputChannel[Any] {
 
   private[actors] var kill = () => {}
 
-  private class ExitSuspendLoop extends Throwable
-
   def suspendActor() {
     isWaiting = true
     while(isWaiting) {
@@ -480,24 +538,21 @@ trait Actor extends OutputChannel[Any] {
     var waittime = msec
     var fromExc = false
     isWaiting = true
-
-    try {
-      while(isWaiting) {
-        try {
-          fromExc = false
-          wait(waittime)
-        } catch {
-          case _: InterruptedException => {
-            fromExc = true
-            val now = Platform.currentTime
-            val waited = now-ts
-            waittime = msec-waited
-            if (waittime < 0) { isWaiting = false }
-          }
+    while(isWaiting) {
+      try {
+        fromExc = false
+        wait(waittime)
+      } catch {
+        case _: InterruptedException => {
+          fromExc = true
+          val now = Platform.currentTime
+          val waited = now-ts
+          waittime = msec-waited
+          if (waittime < 0) { isWaiting = false }
         }
-        if (!fromExc) throw new ExitSuspendLoop
       }
-    } catch { case _: ExitSuspendLoop => }
+      if (!fromExc) { isWaiting = false }
+    }
     // links: check if we should exit
     if (shouldExit) exit()
   }
@@ -557,7 +612,7 @@ trait Actor extends OutputChannel[Any] {
   }
 
   var trapExit = false
-  private[actors] var exitReason: String = "normal"
+  private[actors] var exitReason: AnyRef = 'normal
   private[actors] var exiting = false
   private[actors] var shouldExit = false
 
@@ -569,17 +624,17 @@ trait Actor extends OutputChannel[Any] {
    * <p>
    *   For each linked actor <code>a</code> with
    *   <code>trapExit</code> set to <code>true</code>, send message
-   *   <code>Exit(self, reason)</code> to <code>a</code>.
+   *   <code>{'EXIT, self, reason}</code> to <code>a</code>.
    * </p>
    * <p>
    *   For each linked actor <code>a</code> with
    *   <code>trapExit</code> set to <code>false</code> (default),
    *   call <code>a.exit(reason)</code> if
-   *   <code>!reason.equals("normal")</code>.
+   *   <code>reason != 'normal</code>.
    * </p>
    */
-  def exit(reason: String): Nothing = {
-    kill()
+  def exit(reason: AnyRef): Nothing = {
+    if (reason == 'normal) kill()
     // links
     if (!links.isEmpty) {
       exitReason = reason
@@ -588,14 +643,7 @@ trait Actor extends OutputChannel[Any] {
     throw new ExitActorException
   }
 
-  def exit(): Nothing = {
-    kill()
-    // links
-    if (!links.isEmpty) {
-      exitLinked()
-    }
-    throw new ExitActorException
-  }
+  def exit(): Nothing = exit('normal)
 
   // Assume !links.isEmpty
   private[actors] def exitLinked() {
@@ -610,11 +658,26 @@ trait Actor extends OutputChannel[Any] {
     })
   }
 
+  // Assume !links.isEmpty
+  private[actors] def exitLinked(reason: AnyRef) {
+    exitReason = reason
+    exiting = true
+    // remove this from links
+    links = links.remove(this.==)
+    // exit linked processes
+    links.foreach((linked: Actor) => {
+      unlink(linked)
+      if (!linked.exiting)
+        linked.exit(this, exitReason)
+    })
+  }
+
   // Assume !this.exiting
-  private[actors] def exit(from: Actor, reason: String) {
-    if (trapExit)
-      this ! Exit(from, reason)
-    else if (!reason.equals("normal"))
+  private[actors] def exit(from: Actor, reason: AnyRef) {
+    if (trapExit) {
+      this ! Triple('EXIT, from, reason)
+    }
+    else if (reason != 'normal)
       this.synchronized {
         shouldExit = true
         exitReason = reason
@@ -629,13 +692,33 @@ trait Actor extends OutputChannel[Any] {
 
 
 /**
- * Messages of this type are sent to each actor <code>a</code>
- * that is linked to an actor <code>b</code> whenever
- * <code>b</code> terminates and <code>a</code> has
- * <code>trapExit</code> set to <code>true</code>.
+ * <p>This object is used as the timeout pattern in
+ * <code>receiveWithin</code> and <code>reactWithin</code>.
+ * </p>
+ * <p>The following example demonstrates its usage:</p><pre>
+ * receiveWithin(500) {
+ *   case {x, y}  => ...
+ *   case TIMEOUT => ...
+ * }
+ * </pre>
  *
- * @version 0.9.2
+ * @version 0.9.4
  * @author Philipp Haller
  */
-case class Exit(from: Actor, reason: String)
+case object TIMEOUT
 
+
+/**
+ * <p>This class is used to manage control flow of actor
+ * executions.</p>
+ *
+ * @version 0.9.4
+ * @author Philipp Haller
+ */
+private[actors] class SuspendActorException extends Throwable {
+  /*
+   * For efficiency reasons we do not fill in
+   * the execution stack trace.
+   */
+  override def fillInStackTrace(): Throwable = this
+}
