@@ -6,13 +6,13 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: cloneable.scala 5880 2006-03-02 23:05:17Z mihaylov $
+// $Id: cloneable.scala 9951 2007-02-09 19:09:06Z odersky $
 
 
 package scala;
 
 
 /**
- * An attribute that designates the class to which it is applied as cloneable
+ * An annotation that designates the class to which it is applied as cloneable
  */
-class cloneable extends Attribute {}
+class cloneable extends Annotation {}
