@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Queue.scala 9215 2006-11-10 23:04:57Z mihaylov $
+// $Id: Queue.scala 9980 2007-02-13 22:06:58Z odersky $
 
 
 package scala.collection.immutable
@@ -103,7 +103,7 @@ class Queue[+A](elem: A*) extends Seq[A] {
   def dequeue: Pair[A, Queue[A]] = {
     val Pair(newOut, newIn) =
       if (out.isEmpty) Pair(in.reverse, Nil)
-      else Pair(out, in);
+      else Pair(out, in)
     if (newOut.isEmpty) throw new NoSuchElementException("queue empty")
     else Pair(newOut.head, mkQueue(newIn, newOut.tail))
   }
