@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: HashSet.scala 9621 2007-01-17 14:29:25Z michelou $
+// $Id: HashSet.scala 9803 2007-01-30 14:35:11Z mcdirmid $
 
 package scala.collection.jcl;
 
@@ -14,7 +14,7 @@ package scala.collection.jcl;
  *
  *  @author Sean McDirmid
  */
-class HashSet[A](override val underlying: java.util.HashSet) extends ConcreteWrapper[A] with SetWrapper[A] {
+class HashSet[A](override val underlying: java.util.HashSet) extends CollectionWrapper[A] with SetWrapper[A] {
   /** Creates an underlying Java hash set. */ 
   def this() = this(new java.util.HashSet);
 }
