@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: TreeSet.scala 9621 2007-01-17 14:29:25Z michelou $
+// $Id: TreeSet.scala 9803 2007-01-30 14:35:11Z mcdirmid $
 
 package scala.collection.jcl;
 
@@ -15,6 +15,6 @@ package scala.collection.jcl;
  *
  *  @author Sean McDirmid
  */
-class TreeSet[A <% Ordered[A]] extends ConcreteWrapper[A] with SortedSetWrapper[A] {
+class TreeSet[A <% Ordered[A]] extends CollectionWrapper[A] with SortedSetWrapper[A] {
   val underlying = new java.util.TreeSet(new Comparator[A]);
 }
