@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: DefaultMapModel.scala 9669 2007-01-22 21:04:13Z odersky $
+// $Id: DefaultMapModel.scala 9980 2007-02-13 22:06:58Z odersky $
 
 
 package scala.collection.mutable
@@ -40,6 +40,6 @@ trait DefaultMapModel[A, B] extends Map[A, B] {
     else e.value = value
   }
 
-  def elements = entries map {e => {e.key, e.value}}
+  def elements = entries map {e => Pair(e.key, e.value)}
 }
 
