@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: ImmutableSetAdaptor.scala 8837 2006-10-04 12:18:48Z michelou $
+// $Id: ImmutableSetAdaptor.scala 9670 2007-01-22 22:17:43Z odersky $
 
 
 package scala.collection.mutable
@@ -44,6 +44,6 @@ class ImmutableSetAdaptor[A](protected var set: immutable.Set[A]) extends Set[A]
 
   def -=(elem: A): Unit = { set = set - elem }
 
-  def clear: Unit = { set = set.empty }
+  override def clear(): Unit = { set = set.empty }
 
 }
