@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Map.scala 9669 2007-01-22 21:04:13Z odersky $
+// $Id: Map.scala 9980 2007-02-13 22:06:58Z odersky $
 
 
 package scala.collection.mutable
@@ -28,7 +28,7 @@ import Predef._
 object Map {
 
   /** The empty map of this type; this is implemented as a hashtable */  
-  def empty[A, B] = new HashMap[A, B]
+  def empty[A, B]: Map[A, B] = new HashMap[A, B]
 
   /** The canonical factory for this type
    */
