@@ -12,7 +12,7 @@
 package scala
 
 /**
- * An attribute that designates the definition to which it is applied as deprecated.
+ * An annotation that designates the definition to which it is applied as deprecated.
  * Access to the member then generates a deprecated warning.
  */
-class deprecated extends Attribute {}
+class deprecated extends StaticAnnotation {}
