@@ -6,14 +6,14 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: BoxedObjectArray.scala 9546 2007-01-03 15:56:13Z odersky $
+// $Id: BoxedObjectArray.scala 10044 2007-02-20 11:23:17Z odersky $
 
 
 package scala.runtime
 
 
-import Predef.Class
-import compat.Platform.{createArray, getElementClass}
+import Predef._
+import compat.Platform.createArray
 
 [serializable]
 final class BoxedObjectArray(val value: Array[AnyRef]) extends BoxedArray {
@@ -36,7 +36,7 @@ final class BoxedObjectArray(val value: Array[AnyRef]) extends BoxedArray {
   override def hashCode(): Int = value.hashCode()
 
   private def create(length: Int): Array[AnyRef] = {
-    createArray(getElementClass(value), length).asInstanceOf[Array[AnyRef]]
+    createArray(value.getClass().getComponentType(), length).asInstanceOf[Array[AnyRef]]
   }
 
   override def subArray(start: Int, end: Int): Array[AnyRef] = {
@@ -62,4 +62,11 @@ final class BoxedObjectArray(val value: Array[AnyRef]) extends BoxedArray {
     }
     new BoxedObjectArray(result)
   }
+
+  final override def slice(start: Int, end: Int): BoxedArray = {
+    val len = end - start
+    val result = create(len)
+    Array.copy(value, start, result, 0, len)
+    new BoxedObjectArray(result)
+  }
 }
