@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: RichString.scala 9578 2007-01-09 19:39:31Z odersky $
+// $Id: RichString.scala 9956 2007-02-11 16:44:10Z mihaylov $
 
 
 package scala.runtime
@@ -79,7 +79,7 @@ final class RichString(val self: String) extends Proxy with Seq[Char] with Order
   def linesWithSeparators = new Iterator[String] {
     val len = self.length
     var index = 0
-    def hasNext: Boolean = index <= len
+    def hasNext: Boolean = index < len
     def next: String = {
       if (index >= len) throw new NoSuchElementException("next on empty iterator")
       val start = index
