@@ -1,11 +1,11 @@
 /*    __  ______________                                                      *\
 **   /  |/ / ____/ ____/                                                      **
 **  / | | /___  / /___                                                        **
-** /_/|__/_____/_____/ Copyright 2005-2006 LAMP/EPFL                          **
+** /_/|__/_____/_____/ Copyright 2005-2007 LAMP/EPFL                          **
 **                                                                            **
 \*                                                                            */
 
-// $Id: ScalaTool.scala 9549 2007-01-03 17:20:09Z dubochet $
+// $Id: ScalaTool.scala 9908 2007-02-06 17:15:34Z mihaylov $
 
 package scala.tools.ant {
 
@@ -27,7 +27,7 @@ package scala.tools.ant {
 
   /** <p>
    *    An Ant task that generates a shell or batch script to execute a
-   *    <a href="http://scala.epfl.ch/" target="_top">Scala</a> program.
+   *    <a href="http://scala-lang.org/" target="_top">Scala</a> program.
    *    This task can take the following parameters as attributes:
    *  </p><ul>
    *  <li>file (mandatory),</li>
@@ -334,7 +334,7 @@ package scala.tools.ant {
       if (mainClass.isEmpty) error("Main class must be set.")
       if (platforms.isEmpty) platforms = Platforms.values
       if (name.isEmpty) name = Some(file.get.getName)
-      val patches = ListMap.Empty.
+      val patches = ListMap.empty.
         update("name", name.get).
         update("class", mainClass.get).
         update("version", version).
