@@ -1,8 +1,8 @@
 /* NSC -- new Scala compiler
- * Copyright 2005-2006 LAMP/EPFL
+ * Copyright 2005-2007 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Global.scala 9685 2007-01-23 15:34:40Z spoon $
+// $Id: Global.scala 9996 2007-02-15 16:51:32Z odersky $
 
 package scala.tools.nsc
 
@@ -11,9 +11,8 @@ import java.io.{IOException, FileNotFoundException}
 import java.nio.charset._
 import compat.Platform.currentTime
 import scala.tools.nsc.io.{SourceReader, AbstractFile}
-import scala.tools.nsc.util.ClassPath
-import scala.tools.nsc.util.{Position, SourceFile}
 import scala.tools.nsc.reporters._
+import scala.tools.nsc.util.{ClassPath, Position, SourceFile}
 
 import scala.collection.mutable.{HashSet, HashMap, ListBuffer}
 
@@ -28,6 +27,7 @@ import transform._
 import backend.icode.{ICodes, GenICode, Checkers}
 import backend.ScalaPrimitives
 import backend.jvm.GenJVM
+import backend.msil.GenMSIL
 import backend.opt.{Inliners, ClosureElimination, DeadCodeElimination}
 import backend.icode.analysis._
 
@@ -39,11 +39,11 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
   def this(reporter: Reporter) =
     this(new Settings(err => reporter.error(null,err)), 
          reporter)
-         
+
   def this(settings: Settings) =
-    this(settings, new ConsoleReporter)
-         
-  def this() = this(new Settings, new ConsoleReporter)
+    this(settings, new ConsoleReporter(settings))
+
+  //def this() = this(new Settings, new ConsoleReporter)
   
   // sub-components --------------------------------------------------
   object treePrinters extends TreePrinters {
@@ -76,10 +76,10 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
     val global: Global.this.type = Global.this
   }
 
-/*  object icodeReader extends ICodeReader {
+  object icodeReader extends ICodeReader {
     val global: Global.this.type = Global.this
   }
-*/
+
   object analysis extends TypeFlowAnalysis {
     val global: Global.this.type = Global.this
   }
@@ -150,7 +150,7 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
 
   private val reader: SourceReader = {
     def stdCharset: Charset = {
-      settings.encoding.value = "ISO-8859-1"; // A mandatory charset
+      settings.encoding.value = Properties.encodingString // A mandatory charset
       Charset.forName(settings.encoding.value)
     }
     val charset =
@@ -170,11 +170,11 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
   val classPath0 = new ClassPath(false && onlyPresentation)
 
   val classPath = new classPath0.Build(
-    settings.classpath.value,
+    if (forMSIL) "" else settings.classpath.value,
     settings.sourcepath.value,
     settings.outdir.value,
-    settings.bootclasspath.value,
-    settings.extdirs.value)
+    if (forMSIL) "" else settings.bootclasspath.value,
+    if (forMSIL) "" else settings.extdirs.value)
 
   if (settings.verbose.value) {
     inform("[Classpath = " + classPath+"]")
@@ -201,7 +201,8 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
     val global: Global.this.type = Global.this
   }
 
-  def rootLoader: LazyType = new loaders.PackageLoader(classPath.root /* getRoot() */)
+  def rootLoader: LazyType = if (forMSIL) new loaders.NamespaceLoader(classPath.root)
+                             else new loaders.PackageLoader(classPath.root /* getRoot() */)
 
   val migrateMsg = "migration problem when moving from Scala version 1.0 to version 2.0:\n"
 
@@ -211,7 +212,7 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
 
   val MaxPhases = 64
 
-  val phaseWithId = new Array[Phase](MaxPhases)
+  val phaseWithId = new Array[Phase](MaxPhases);
   { for (val i <- List.range(0, MaxPhases)) phaseWithId(i) = NoPhase }
 
   abstract class GlobalPhase(prev: Phase) extends Phase(prev) {
@@ -343,6 +344,10 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
     val global: Global.this.type = Global.this
   }
 
+  object genMSIL extends GenMSIL {
+    val global: Global.this.type = Global.this
+  }
+
   object icodeChecker extends checkers.ICodeChecker()
 
   object typer extends analyzer.Typer(
@@ -358,7 +363,6 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
     uncurry,
     tailCalls,
     explicitOuter,
-    //transMatcher,
 //    checkDefined,
     erasure,
     lambdaLift,
@@ -371,7 +375,7 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
     inliner,
     closureElimination,
     deadCode,
-    genJVM,
+    if (forMSIL) genMSIL else genJVM,
     sampleTransform)
 
   protected def insertBefore(c: SubComponent, cs: List[SubComponent], before: SubComponent): List[SubComponent] = cs match {
@@ -508,7 +512,7 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
           if (sym.isTerm) sym.moduleClass.reset(loaders.moduleClassLoader)
         }
       } else {
-        assert(symData.isEmpty || !settings.stop.value.isEmpty || !settings.skip.value.isEmpty)
+        //assert(symData.isEmpty || !settings.stop.value.isEmpty || !settings.skip.value.isEmpty, symData)
         if (deprecationWarnings) {
           warning("there were deprecation warnings; re-run with -deprecation for details")
         }
@@ -551,7 +555,7 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
         if (settings.Xscript.value && filenames.length != 1)
           error("can only compile one script at a time")
         val sources = filenames map (
-          if (settings.Xscript.value) {x => ScriptRunner.wrappedScript(x, &Global.this.getSourceFile)} else getSourceFile)
+          /* if (settings.Xscript.value) {x => ScriptRunner.wrappedScript(x, &Global.this.getSourceFile)} else */ getSourceFile)
         compileSources(sources)
       } catch {
         case ex: IOException => error(ex.getMessage())
@@ -633,6 +637,7 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
   }
 
   def forCLDC: Boolean = settings.target.value == "cldc"
+  def forMSIL: Boolean = settings.target.value == "msil"
   def onlyPresentation = settings.doc.value
   // position stuff
   final val positionConfiguration: PositionConfiguration = initConfig;
@@ -650,7 +655,7 @@ class Global(var settings: Settings, var reporter: Reporter) extends SymbolTable
     val posConfig : PositionConfiguration = positionConfiguration;
     posConfig.FirstPos.asInstanceOf[PositionType];
   }
-  final val NoPos = positionConfiguration.NoPos
+  final def NoPos = positionConfiguration.NoPos
   final def coerceIntToPos(pos: Int): PositionType =
     positionConfiguration.coerceIntToPos(pos)
   implicit final def coercePosToInt(pos: PositionType): Int =
