@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: TreeBrowsers.scala 9411 2006-11-29 12:17:38Z emir $
+// $Id: TreeBrowsers.scala 10036 2007-02-19 13:56:55Z odersky $
 
 package scala.tools.nsc.ast
 
@@ -260,7 +260,7 @@ abstract class TreeBrowsers {
       case DocDef(comment, definition) =>
         Pair("DocDef", EMPTY)
 
-      case ClassDef(mods, name, tparams, tpt, impl) =>
+      case ClassDef(mods, name, tparams, self, impl) =>
         Pair("ClassDef", name)
 
       case PackageDef(packaged, impl) =>
@@ -353,6 +353,12 @@ abstract class TreeBrowsers {
       case TypeTree() =>
         Pair("TypeTree", EMPTY)
 
+      case Annotated(constr, elements, arg) =>
+        Pair("Annotated", EMPTY)
+            
+      case AttributedTypeTree(attribs, tpt) =>
+        Pair("AttributedTypeTree", EMPTY)
+            
       case SingletonTypeTree(ref) =>
         Pair("SingletonType", EMPTY)
 
@@ -389,10 +395,10 @@ abstract class TreeBrowsers {
       case DocDef(comment, definition) =>
         List(definition)
 
-      case ClassDef(mods, name, tparams, tpt, impl) => {
+      case ClassDef(mods, name, tparams, self, impl) => {
         var children: List[Tree] = List()
         children = tparams ::: children
-        tpt :: impl :: children
+        self :: impl :: children
       }
 
       case PackageDef(name, stats) =>
@@ -494,6 +500,9 @@ abstract class TreeBrowsers {
       case TypeTree() =>
         Nil
 
+      case Annotated(constr, elements, arg) =>
+        constr :: elements ::: List(arg)
+        
       case SingletonTypeTree(ref) =>
         List(ref)
 
@@ -644,6 +653,13 @@ abstract class TreeBrowsers {
                                        "), ") :/:
                         toDocument(result) :: ")")
         )
+        
+      case AttributedType(attribs, tp) =>
+        Document.group(
+          Document.nest(4, "AttributedType(" :/:
+                        attribs.mkString("[", ",", "]") :/:
+                        "," :/: toDocument(tp) :: ")")
+        )
 
       case _ =>
         abort("Unknown case: " + t.toString())
