@@ -2,7 +2,7 @@
  * Copyright 2005-2007 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Trees.scala 9412 2006-11-29 12:17:57Z emir $
+// $Id: Trees.scala 10044 2007-02-20 11:23:17Z odersky $
 
 package scala.tools.nsc.ast
 
@@ -17,7 +17,9 @@ trait Trees requires Global {
   //statistics
   var nodeCount = 0
 
-  case class Modifiers(flags: int, privateWithin: Name) {
+  case class Modifiers(flags: int, privateWithin: Name, attributes: List[Annotation]) {
+    def isCovariant     = hasFlag(COVARIANT    )
+    def isContravariant = hasFlag(CONTRAVARIANT)
     def isPrivate   = hasFlag(PRIVATE  )
     def isProtected = hasFlag(PROTECTED)
     def isVariable  = hasFlag(MUTABLE  )
@@ -36,22 +38,24 @@ trait Trees requires Global {
     def & (flag: Int): Modifiers = {
       val flags1 = flags & flag
       if (flags1 == flags) this
-      else Modifiers(flags1, privateWithin) setAttr attributes
+      else Modifiers(flags1, privateWithin, attributes)
     }
     def &~ (flag: Int): Modifiers = {
       val flags1 = flags & (~flag)
       if (flags1 == flags) this
-      else Modifiers(flags1, privateWithin) setAttr attributes
+      else Modifiers(flags1, privateWithin, attributes)
     }
     def | (flag: int): Modifiers = {
       val flags1 = flags | flag
       if (flags1 == flags) this
-      else Modifiers(flags1, privateWithin) setAttr attributes
+      else Modifiers(flags1, privateWithin, attributes)
     }
-    def setAttr(attrs: List[Attribute]): this.type = {attributes = attrs; this }
-    var attributes: List[Attribute] = List()
+    def withAnnotations(attrs: List[Annotation]) = 
+      if (attrs.isEmpty) this
+      else Modifiers(flags, privateWithin, attributes ::: attrs)
   }
 
+  def Modifiers(flags: int, privateWithin: Name): Modifiers = Modifiers(flags, privateWithin, List())
   def Modifiers(flags: int): Modifiers = Modifiers(flags, nme.EMPTY.toTypeName)
   def Modifiers(flags: long): Modifiers = Modifiers(flags.asInstanceOf[int])
 
@@ -98,36 +102,33 @@ trait Trees requires Global {
       case t: Tree => this eq t
       case _ => false
     }
-
-    def equalsStructure(that: Tree): Boolean = if (this == that) true else (this:Any) match {
-      case thiz : Product if ((that ne null) && thiz.getClass == that.getClass) =>
-        val that0 = that.asInstanceOf[Product]
-        val result: Iterator[Boolean] =
-          for (val i <- 0.until(thiz.arity)) yield thiz.element(i) match {
-            case tree: Tree =>
-              val b = tree.equalsStructure(that0.element(i).asInstanceOf[Tree])
-              b
-            case list: List[_] if (that0.element(i).isInstanceOf[List[Any]]) =>
-              val listThat = that0.element(i).asInstanceOf[List[Any]]
-              if (list.length == listThat.length) (for (val x <- list.zip(listThat)) yield {
-                if ((null != x._1) && x._1.isInstanceOf[Tree] && x._2.isInstanceOf[Tree]) {
-                  val b = x._1.asInstanceOf[Tree] equalsStructure x._2.asInstanceOf[Tree]
-                  b
-                } else x._1 == x._2
-              }).foldLeft(true)((x,y) => x && y)
-              else false
-            case elem =>
-              val b = elem == that0.element(i)
-              b
-          }
-          val b = result.foldLeft(true)((x,y) => x && y)
-          if (b) {
-            tpe == that.tpe
-          } else false
-      case _ =>
-        false
+    def equalsStructure(that: Tree): Boolean = {
+      val this0 = this.asInstanceOf[Product];
+      val that0 = that.asInstanceOf[Product];
+      if (this == that) return true;
+      if (this.getClass != that.getClass) return false;
+      assert(this0.arity == that0.arity);
+      def equals0(thiz : Any, that : Any) : Boolean = thiz match {
+      case thiz : Tree => 
+        thiz.equalsStructure(that.asInstanceOf[Tree]);
+      case thiz : List[_] =>
+        val that0 = that.asInstanceOf[List[Any]];
+        if (thiz.length != that0.length) false;
+        else {
+          val results0 = for (val i <- 0.until(thiz.length).toList) 
+            yield equals0(thiz(i), that0(i));  
+          results0.foldLeft(true)((x,y) => x && y);
+        }
+      case thiz => thiz == that;
+      }
+      val results = for (val i <- 0.until(this0.arity).toList) yield 
+        equals0(this0.element(i), that0.element(i));
+      val b = results.foldLeft(true)((x,y) => x && y);
+      if (!b) return false;
+      return if (tpe == null || tpe == NoType) {
+        that.tpe == null || that.tpe == NoType;
+      } else tpe == that.tpe
     }
-
     def duplicate: this.type =
       (duplicator transform this).asInstanceOf[this.type]
 
@@ -137,6 +138,9 @@ trait Trees requires Global {
       if (hasSymbol) symbol = tree.symbol
       this
     }
+    
+    def withAttributes(attribs: List[Tree]) = 
+      AttributedTypeTree(attribs, this)
   }
 
   trait SymTree extends Tree {
@@ -153,6 +157,8 @@ trait Trees requires Global {
     override def isTerm = true
   }
 
+  /** A tree for a type.  Note that not all type trees implement
+    * this trait; in particular, Ident's are an exception. */
   trait TypTree extends Tree {
     override def isType = true
   }
@@ -163,20 +169,19 @@ trait Trees requires Global {
     override val copy = new StrictTreeCopier
   }
 
-  private def syntheticParams(owner: Symbol, formals: List[Type]): List[Symbol] = {
+  private def syntheticParams(owner: Symbol, mtp: Type): List[List[Symbol]] = {
     var cnt = 0
     def freshName() = { cnt = cnt + 1; newTermName("x$" + cnt) }
-    for (val formal <- formals) yield
-      owner.newValueParameter(owner.pos, freshName()).setInfo(formal)
-  }
-
-  private def syntheticParams(owner: Symbol, mtp: Type): List[List[Symbol]] = mtp match {
-    case PolyType(_, restp) =>
-      syntheticParams(owner, restp)
-    case MethodType(formals, restp) =>
-      syntheticParams(owner, formals) :: syntheticParams(owner, restp)
-    case _ =>
-      List()
+    def synthetics(mtp: Type): List[List[Symbol]] = mtp match {
+      case PolyType(_, restp) =>
+        synthetics(restp)
+      case MethodType(formals, restp) =>
+        (formals map (f => owner.newValueParameter(owner.pos, freshName()).setInfo(f))) ::
+          synthetics(restp)
+      case _ =>
+        List()
+    }
+    synthetics(mtp)
   }
 
 //  def nextPhase = if (phase.id > globalPhase.id) phase else phase.next;
@@ -217,7 +222,7 @@ trait Trees requires Global {
   }
 
   /** Class definition */
-  case class ClassDef(mods: Modifiers, name: Name, tparams: List[AbsTypeDef], tpt: Tree, impl: Template)
+  case class ClassDef(mods: Modifiers, name: Name, tparams: List[AbsTypeDef], self: ValDef, impl: Template)
        extends ImplDef
 
   /**
@@ -230,7 +235,7 @@ trait Trees requires Global {
       ClassDef(Modifiers(sym.flags),
                sym.name,
                sym.typeParams map AbsTypeDef,
-               if (sym.thisSym == sym) EmptyTree else TypeTree(sym.typeOfThis),
+               if (sym.thisSym == sym || phase.erasedTypes) emptyValDef else ValDef(sym.thisSym),
                impl) setSymbol sym
     }
 
@@ -245,8 +250,8 @@ trait Trees requires Global {
    *                   and value parameter fields.
    *  @return          ...
    */
-  def ClassDef(sym: Symbol, vparamss: List[List[ValDef]], argss: List[List[Tree]], body: List[Tree]): ClassDef =
-    ClassDef(sym, Template(sym.info.parents map TypeTree, vparamss, argss, body))
+  def ClassDef(sym: Symbol, constrMods: Modifiers, vparamss: List[List[ValDef]], argss: List[List[Tree]], body: List[Tree]): ClassDef =
+    ClassDef(sym, Template(sym.info.parents map TypeTree, constrMods, vparamss, argss, body))
 
   /** Singleton object definition
    *
@@ -293,6 +298,8 @@ trait Trees requires Global {
 
   def ValDef(sym: Symbol): ValDef = ValDef(sym, EmptyTree)
 
+  object emptyValDef extends ValDef(Modifiers(PRIVATE), nme.WILDCARD, TypeTree(NoType), EmptyTree)
+
   /** Method definition
    *
    *  @param mods
@@ -309,7 +316,7 @@ trait Trees requires Global {
     assert(rhs.isTerm)
   }
 
-  def DefDef(sym: Symbol, vparamss: List[List[ValDef]], rhs: Tree): DefDef =
+  def DefDef(sym: Symbol, mods: Modifiers, vparamss: List[List[ValDef]], rhs: Tree): DefDef =
     posAssigner.atPos(sym.pos) {
       assert(sym != NoSymbol)
       DefDef(Modifiers(sym.flags),
@@ -320,11 +327,17 @@ trait Trees requires Global {
              rhs) setSymbol sym
     }
 
-  def DefDef(sym: Symbol, rhs: List[List[Symbol]] => Tree): DefDef = {
+  def DefDef(sym: Symbol, vparamss: List[List[ValDef]], rhs: Tree): DefDef =
+    DefDef(sym, Modifiers(sym.flags), vparamss, rhs)
+
+  def DefDef(sym: Symbol, mods: Modifiers, rhs: List[List[Symbol]] => Tree): DefDef = {
     val vparamss = syntheticParams(sym, sym.tpe)
-    DefDef(sym, vparamss map (.map(ValDef)), rhs(vparamss))
+    DefDef(sym, mods, vparamss map (.map(ValDef)), rhs(vparamss))
   }
 
+  def DefDef(sym: Symbol, rhs: List[List[Symbol]] => Tree): DefDef = 
+    DefDef(sym, Modifiers(sym.flags), rhs)
+
   /** Abstract type or type parameter
    *
    *  @param mods
@@ -399,8 +412,8 @@ trait Trees requires Global {
   case class Import(expr: Tree, selectors: List[Pair[Name, Name]])
        extends SymTree
 
-  /** Attribute application (constructor arguments + name-value pairs) */
-  case class Attribute(constr: Tree, elements: List[Tree])
+  /** Annotation application (constructor arguments + name-value pairs) */
+  case class Annotation(constr: Tree, elements: List[Tree])
        extends TermTree
 
   /** Documented definition, eliminated by analyzer */
@@ -427,18 +440,23 @@ trait Trees requires Global {
    *  @param body        ...
    *  @return            ...
    */
-  def Template(parents: List[Tree], vparamss: List[List[ValDef]], argss: List[List[Tree]], body: List[Tree]): Template = {
+  def Template(parents: List[Tree], constrMods: Modifiers, vparamss: List[List[ValDef]], argss: List[List[Tree]], body: List[Tree]): Template = {
     /** Add constructor to template */
     var vparamss1 =
-      vparamss map (.map (vd =>
-        ValDef(Modifiers(vd.mods.flags & IMPLICIT | PARAM) setAttr vd.mods.attributes,
-               vd.name, vd.tpt.duplicate, EmptyTree)));
+      vparamss map (.map (vd => {
+        val ret = ValDef(Modifiers(vd.mods.flags & IMPLICIT | PARAM) withAnnotations vd.mods.attributes,
+            vd.name, vd.tpt.duplicate, EmptyTree).setPos(vd.pos)
+        if (false/*inIDE*/ && vd.symbol != NoSymbol) 
+          ret.symbol = vd.symbol
+        ret
+       }));
     if (vparamss1.isEmpty ||
         !vparamss1.head.isEmpty && (vparamss1.head.head.mods.flags & IMPLICIT) != 0) 
       vparamss1 = List() :: vparamss1;
     val superRef: Tree = Select(Super(nme.EMPTY.toTypeName, nme.EMPTY.toTypeName), nme.CONSTRUCTOR)
     val superCall = posAssigner.atPos(parents.head.pos) { (superRef /: argss) (Apply) }
-    val constr: Tree = DefDef(NoMods, nme.CONSTRUCTOR, List(), vparamss1, TypeTree(), superCall)
+    val constr: Tree = 
+      DefDef(constrMods, nme.CONSTRUCTOR, List(), vparamss1, TypeTree(), superCall)
     Template(parents, List.flatten(vparamss) ::: constr :: body)
   }
 
@@ -623,7 +641,11 @@ trait Trees requires Global {
   def Literal(value: Any): Literal =
     Literal(Constant(value))
 
-  /** General type term, introduced by the phase <code>RefCheck</code>. */
+  /** A synthetic term holding an arbitrary type.  Not to be confused with
+    * with TypTree, the trait for trees that are only used for type trees.
+    * TypeTree's are inserted in several places, but most notably in
+    * <code>RefCheck</code>, where the arbitrary type trees are all replaced by
+    * TypeTree's. */
   case class TypeTree() extends TypTree {
     var original: Tree = _
 
@@ -631,12 +653,24 @@ trait Trees requires Global {
       original = tree
       setPos(tree.pos)
     }
+    
     override def isEmpty = (tpe eq null) || tpe == NoType
   }
 
   def TypeTree(tp: Type): TypeTree = TypeTree() setType tp
   // def TypeTree(tp: Type, tree : Tree): TypeTree = TypeTree(tree) setType tp
 
+  /** A tree that has anP attribute attached to it */
+  case class Annotated(constr: Tree, elements: List[Tree], arg: Tree) extends Tree {
+    override def isType = arg.isType
+  }
+
+  /** A type tree that has attributes attached to it */
+  case class AttributedTypeTree(attribs: List[Tree], tpt: Tree)
+        extends TypTree {
+    override def withAttributes(attribs: List[Tree]) = 
+      AttributedTypeTree(attribs:::this.attribs, this)
+  }
 
   /** Singleton type, eliminated by RefCheck */
   case class SingletonTypeTree(ref: Tree)
@@ -663,7 +697,7 @@ trait Trees requires Global {
 /* A standard pattern match
   case EmptyTree =>
   case PackageDef(name, stats) =>
-  case ClassDef(mods, name, tparams, tpt, impl) =>
+  case ClassDef(mods, name, tparams, self, impl) =>
   case ModuleDef(mods, name, impl) =>                         (eliminated by refcheck)
   case ValDef(mods, name, tpt, rhs) =>
   case DefDef(mods, name, tparams, vparamss, tpt, rhs) =>
@@ -671,7 +705,7 @@ trait Trees requires Global {
   case AliasTypeDef(mods, name, tparams, rhs) =>                  (eliminated by erasure)
   case LabelDef(name, params, rhs) =>
   case Import(expr, selectors) =>                                 (eliminated by typecheck)
-  case Attribute(constr, elements) =>                             (eliminated by typecheck)
+  case Annotation(constr, elements) =>                             (eliminated by typecheck)
   case DocDef(comment, definition) =>                             (eliminated by typecheck)
   case Template(parents, body) =>
   case Block(stats, expr) =>
@@ -698,7 +732,9 @@ trait Trees requires Global {
   case Select(qualifier, selector) =>
   case Ident(name) =>
   case Literal(value) =>
-  case TypeTree() =>
+  case TypeTree() =>                                              (introduced by refcheck)
+  case Annotated(constr, elements, arg) =>                       (eliminated by typer)
+  case AttributedTypeTree(attribs, tpt) =>                        (eliminated by uncurry)
   case SingletonTypeTree(ref) =>                                  (eliminated by uncurry)
   case SelectFromTypeTree(qualifier, selector) =>                 (eliminated by uncurry)
   case CompoundTypeTree(templ: Template) =>                       (eliminated by uncurry)
@@ -707,7 +743,7 @@ trait Trees requires Global {
 */
 
   abstract class TreeCopier {
-    def ClassDef(tree: Tree, mods: Modifiers, name: Name, tparams: List[AbsTypeDef], tpt: Tree, impl: Template): ClassDef
+    def ClassDef(tree: Tree, mods: Modifiers, name: Name, tparams: List[AbsTypeDef], self: ValDef, impl: Template): ClassDef
     def PackageDef(tree: Tree, name: Name, stats: List[Tree]): PackageDef
     def ModuleDef(tree: Tree, mods: Modifiers, name: Name, impl: Template): ModuleDef
     def ValDef(tree: Tree, mods: Modifiers, name: Name, tpt: Tree, rhs: Tree): ValDef
@@ -716,7 +752,7 @@ trait Trees requires Global {
     def AliasTypeDef(tree: Tree, mods: Modifiers, name: Name, tparams: List[AbsTypeDef], rhs: Tree): AliasTypeDef
     def LabelDef(tree: Tree, name: Name, params: List[Ident], rhs: Tree): LabelDef
     def Import(tree: Tree, expr: Tree, selectors: List[Pair[Name, Name]]): Import
-    def Attribute(tree: Tree, constr: Tree, elements: List[Tree]): Attribute
+    def Annotation(tree: Tree, constr: Tree, elements: List[Tree]): Annotation
     def DocDef(tree: Tree, comment: String, definition: Tree): DocDef
     def Template(tree: Tree, parents: List[Tree], body: List[Tree]): Template
     def Block(tree: Tree, stats: List[Tree], expr: Tree): Block
@@ -744,6 +780,8 @@ trait Trees requires Global {
     def Ident(tree: Tree, name: Name): Ident
     def Literal(tree: Tree, value: Constant): Literal
     def TypeTree(tree: Tree): TypeTree
+    def Annotated(tree: Tree, constr: Tree, elements: List[Tree], arg: Tree): Annotated
+    def AttributedTypeTree(tree: Tree, attribs: List[Tree], tpt: Tree): AttributedTypeTree
     def SingletonTypeTree(tree: Tree, ref: Tree): SingletonTypeTree
     def SelectFromTypeTree(tree: Tree, qualifier: Tree, selector: Name): SelectFromTypeTree
     def CompoundTypeTree(tree: Tree, templ: Template): CompoundTypeTree
@@ -752,8 +790,8 @@ trait Trees requires Global {
   }
 
   class StrictTreeCopier extends TreeCopier {
-    def ClassDef(tree: Tree, mods: Modifiers, name: Name, tparams: List[AbsTypeDef], tpt: Tree, impl: Template) =
-      new ClassDef(mods, name, tparams, tpt, impl).copyAttrs(tree);
+    def ClassDef(tree: Tree, mods: Modifiers, name: Name, tparams: List[AbsTypeDef], self: ValDef, impl: Template) =
+      new ClassDef(mods, name, tparams, self, impl).copyAttrs(tree);
     def PackageDef(tree: Tree, name: Name, stats: List[Tree]) =
       new PackageDef(name, stats).copyAttrs(tree)
     def ModuleDef(tree: Tree, mods: Modifiers, name: Name, impl: Template) =
@@ -770,8 +808,8 @@ trait Trees requires Global {
       new LabelDef(name, params, rhs).copyAttrs(tree)
     def Import(tree: Tree, expr: Tree, selectors: List[Pair[Name, Name]]) =
       new Import(expr, selectors).copyAttrs(tree)
-    def Attribute(tree: Tree, constr: Tree, elements: List[Tree]) =
-      new Attribute(constr, elements)
+    def Annotation(tree: Tree, constr: Tree, elements: List[Tree]) =
+      new Annotation(constr, elements)
     def DocDef(tree: Tree, comment: String, definition: Tree) =
       new DocDef(comment, definition).copyAttrs(tree)
     def Template(tree: Tree, parents: List[Tree], body: List[Tree]) =
@@ -826,6 +864,10 @@ trait Trees requires Global {
       new Literal(value).copyAttrs(tree)
     def TypeTree(tree: Tree) =
       new TypeTree().copyAttrs(tree)
+    def Annotated(tree: Tree, constr: Tree, elements: List[Tree], arg: Tree) =
+      new Annotated(constr, elements, arg)
+    def AttributedTypeTree(tree: Tree, attribs: List[Tree], tpt: Tree) =
+      new AttributedTypeTree(attribs, tpt)
     def SingletonTypeTree(tree: Tree, ref: Tree) =
       new SingletonTypeTree(ref).copyAttrs(tree)
     def SelectFromTypeTree(tree: Tree, qualifier: Tree, selector: Name) =
@@ -840,10 +882,10 @@ trait Trees requires Global {
 
   class LazyTreeCopier(copy: TreeCopier) extends TreeCopier {
     def this() = this(new StrictTreeCopier)
-    def ClassDef(tree: Tree, mods: Modifiers, name: Name, tparams: List[AbsTypeDef], tpt: Tree, impl: Template) = tree match {
-      case t @ ClassDef(mods0, name0, tparams0, tpt0, impl0)
-      if (mods0 == mods && (name0 == name) && (tparams0 == tparams) && (tpt0 == tpt) && (impl0 == impl)) => t
-      case _ => copy.ClassDef(tree, mods, name, tparams, tpt, impl)
+    def ClassDef(tree: Tree, mods: Modifiers, name: Name, tparams: List[AbsTypeDef], self: ValDef, impl: Template) = tree match {
+      case t @ ClassDef(mods0, name0, tparams0, self0, impl0)
+      if (mods0 == mods && (name0 == name) && (tparams0 == tparams) && (self0 == self) && (impl0 == impl)) => t
+      case _ => copy.ClassDef(tree, mods, name, tparams, self, impl)
     }
     def PackageDef(tree: Tree, name: Name, stats: List[Tree]) = tree match {
       case t @ PackageDef(name0, stats0)
@@ -886,10 +928,10 @@ trait Trees requires Global {
       if (expr0 == expr) && (selectors0 == selectors) => t
       case _ => copy.Import(tree, expr, selectors)
     }
-    def Attribute(tree: Tree, constr: Tree, elements: List[Tree]) = tree match {
-      case t @ Attribute(constr0, elements0)
+    def Annotation(tree: Tree, constr: Tree, elements: List[Tree]) = tree match {
+      case t @ Annotation(constr0, elements0)
       if (constr0 == constr) && (elements0 == elements) => t
-      case _ => copy.Attribute(tree, constr, elements)
+      case _ => copy.Annotation(tree, constr, elements)
     }
     def DocDef(tree: Tree, comment: String, definition: Tree) = tree match {
       case t @ DocDef(comment0, definition0)
@@ -1025,6 +1067,16 @@ trait Trees requires Global {
       case t @ TypeTree() => t
       case _ => copy.TypeTree(tree)
     }
+    def Annotated(tree: Tree, constr: Tree, elements: List[Tree], arg: Tree) = tree match {
+      case t @ Annotated(constr0, elements0, arg0)
+      if (constr0==constr) && (elements0==elements) && (arg0==arg) => t
+      case _ => copy.Annotated(tree, constr, elements, arg)
+    }
+    def AttributedTypeTree(tree: Tree, attribs: List[Tree], tpt: Tree) = tree match {
+      case t @ AttributedTypeTree(attribs0, tpt0)
+      if (attribs0==attribs) && (tpt0==tpt) => t
+      case _ => copy.AttributedTypeTree(tree, attribs, tpt)
+    }
     def SingletonTypeTree(tree: Tree, ref: Tree) = tree match {
       case t @ SingletonTypeTree(ref0)
       if ref0 == ref => t
@@ -1065,22 +1117,24 @@ trait Trees requires Global {
         atOwner(tree.symbol.moduleClass) {
           copy.PackageDef(tree, name, transformStats(stats, currentOwner))
         }
-      case ClassDef(mods, name, tparams, tpt, impl) =>
+      case ClassDef(mods, name, tparams, self, impl) =>
         atOwner(tree.symbol) {
-          copy.ClassDef(tree, mods, name, transformAbsTypeDefs(tparams),
-                        transform(tpt), transformTemplate(impl))
+          copy.ClassDef(tree, mods, name, 
+                        transformAbsTypeDefs(tparams),
+                        if (self ne emptyValDef) transformValDef(self) else self,
+                        transformTemplate(impl))
         }
       case ModuleDef(mods, name, impl) =>
-      atOwner(tree.symbol.moduleClass) {
-        copy.ModuleDef(tree, mods, name, transformTemplate(impl))
+        atOwner(tree.symbol.moduleClass) {
+          copy.ModuleDef(tree, mods, name, transformTemplate(impl))
         }
       case ValDef(mods, name, tpt, rhs) =>
-      atOwner(tree.symbol) {
-        copy.ValDef(tree, mods, name, transform(tpt), transform(rhs))
+        atOwner(tree.symbol) {
+          copy.ValDef(tree, mods, name, transform(tpt), transform(rhs))
         }
       case DefDef(mods, name, tparams, vparamss, tpt, rhs) =>
-      atOwner(tree.symbol) {
-        copy.DefDef(
+        atOwner(tree.symbol) {
+          copy.DefDef(
             tree, mods, name, transformAbsTypeDefs(tparams), transformValDefss(vparamss), transform(tpt), transform(rhs))
         }
       case AbsTypeDef(mods, name, lo, hi) =>
@@ -1095,8 +1149,8 @@ trait Trees requires Global {
         copy.LabelDef(tree, name, transformIdents(params), transform(rhs)) //bq: Martin, once, atOwner(...) works, also change `LamdaLifter.proxy'
       case Import(expr, selectors) =>
         copy.Import(tree, transform(expr), selectors)
-      case Attribute(constr, elements) =>
-        copy.Attribute(tree, transform(constr), transformTrees(elements))
+      case Annotation(constr, elements) =>
+        copy.Annotation(tree, transform(constr), transformTrees(elements))
       case DocDef(comment, definition) =>
         copy.DocDef(tree, comment, transform(definition))
       case Template(parents, body) =>
@@ -1114,7 +1168,7 @@ trait Trees requires Global {
       case Bind(name, body) =>
         copy.Bind(tree, name, transform(body))
       case UnApply(fun, args) =>
-        copy.UnApply(tree, fun, args)
+        copy.UnApply(tree, fun, transformTrees(args)) // bq: see test/.../unapplyContexts2.scala
       case ArrayValue(elemtpt, trees) =>
         copy.ArrayValue(tree, transform(elemtpt), transformTrees(trees))
       case Function(vparams, body) =>
@@ -1153,6 +1207,10 @@ trait Trees requires Global {
         copy.Literal(tree, value)
       case TypeTree() =>
         copy.TypeTree(tree)
+      case Annotated(constr, elements, arg) =>
+        copy.Annotated(tree, transform(constr), transformTrees(elements), transform(arg))
+      case AttributedTypeTree(attribs, tpt) =>
+        copy.AttributedTypeTree(tree, transformTrees(attribs), transform(tpt))
       case SingletonTypeTree(ref) =>
         copy.SingletonTypeTree(tree, transform(ref))
       case SelectFromTypeTree(qualifier, selector) =>
@@ -1171,10 +1229,12 @@ trait Trees requires Global {
       transform(tree: Tree).asInstanceOf[Template]
     def transformAbsTypeDefs(trees: List[AbsTypeDef]): List[AbsTypeDef] =
       List.mapConserve(trees)(tree => transform(tree).asInstanceOf[AbsTypeDef])
+    def transformValDef(tree: ValDef): ValDef =
+      transform(tree).asInstanceOf[ValDef]
     def transformValDefs(trees: List[ValDef]): List[ValDef] =
-      List.mapConserve(trees)(tree => transform(tree).asInstanceOf[ValDef])
+      List.mapConserve(trees)(transformValDef)
     def transformValDefss(treess: List[List[ValDef]]): List[List[ValDef]] =
-      List.mapConserve(treess)(tree => transformValDefs(tree))
+      List.mapConserve(treess)(transformValDefs)
     def transformCaseDefs(trees: List[CaseDef]): List[CaseDef] =
       List.mapConserve(trees)(tree => transform(tree).asInstanceOf[CaseDef])
     def transformIdents(trees: List[Ident]): List[Ident] =
@@ -1203,9 +1263,11 @@ trait Trees requires Global {
         atOwner(tree.symbol.moduleClass) {
           traverseTrees(stats)
         }
-      case ClassDef(mods, name, tparams, tpt, impl) =>
+      case ClassDef(mods, name, tparams, self, impl) =>
         atOwner(tree.symbol) {
-          traverseTrees(tparams); traverse(tpt); traverse(impl)
+          traverseTrees(tparams); 
+          if (self ne emptyValDef) traverse(self); 
+          traverse(impl)
         }
       case ModuleDef(mods, name, impl) =>
         atOwner(tree.symbol.moduleClass) {
@@ -1231,8 +1293,10 @@ trait Trees requires Global {
         traverseTrees(params); traverse(rhs)
       case Import(expr, selectors) =>
         traverse(expr)
-      case Attribute(constr, elements) =>
+      case Annotation(constr, elements) =>
         traverse(constr); traverseTrees(elements)
+      case Annotated(constr, elements, arg) =>
+        traverse(constr); traverseTrees(elements); traverse(arg)
       case DocDef(comment, definition) =>
         traverse(definition) 
       case Template(parents, body) =>
@@ -1289,6 +1353,8 @@ trait Trees requires Global {
         ;
       case TypeTree() =>
         ;
+      case AttributedTypeTree(attribs, tpt) =>
+        traverseTrees(attribs); traverse(tpt)
       case SingletonTypeTree(ref) =>
         traverse(ref)
       case SelectFromTypeTree(qualifier, selector) =>
