@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author Burak Emir
  */
-// $Id: MarkupParsers.scala 9504 2006-12-12 11:27:07Z odersky $
+// $Id: MarkupParsers.scala 9980 2007-02-13 22:06:58Z odersky $
 
 package scala.tools.nsc.ast.parser
 
@@ -575,7 +575,7 @@ class MarkupParser(unit: CompilationUnit, s: Scanner, p: Parser, presWS: boolean
    */
   def xScalaPatterns: List[Tree] = {
     sync;
-    val b = p.patterns(true, false);
+    val b = p.patterns(true);
     if (/*s.*/token != RBRACE) {
       reportSyntaxError(" expected end of Scala patterns");
     }
