@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Scanners.scala 9605 2007-01-14 19:44:59Z spoon $
+// $Id: Scanners.scala 9969 2007-02-12 15:53:42Z odersky $
 
 package scala.tools.nsc.ast.parser
 
@@ -183,15 +183,8 @@ trait Scanners requires SyntaxAnalyzer {
         next.copyFrom(this)
         pos = in.lineStartPos
         if (settings.migrate.value) newNewLine = lastToken != RBRACE && token != EOF;
-        token = NEWLINE
-
-/*
-      } else if (lastToken == RBRACE) {
-        System.out.println("failing to insert NL after RBRACE: " + sepRegions + " " + 
-                           lastPos + " " + in.lineStartPos + " " + pos)
-*/
+        token = if (in.lastBlankLinePos > lastPos) NEWLINES else NEWLINE
       }
-//    System.out.println("token: " + toString());//DEBUG
     }
 
     private def afterLineEnd() = (
@@ -450,8 +443,8 @@ trait Scanners requires SyntaxAnalyzer {
 
     def inFirstOfStat(token: int) = token match {
       case EOF | CASE | CATCH | ELSE | EXTENDS | FINALLY | MATCH | REQUIRES | WITH | YIELD |
-           COMMA | SEMI | NEWLINE | DOT | USCORE | COLON | EQUALS | ARROW |
-           LARROW | SUBTYPE | VIEWBOUND | SUPERTYPE | HASH | AT | 
+           COMMA | SEMI | NEWLINE | NEWLINES | DOT | USCORE | COLON | EQUALS | ARROW |
+           LARROW | SUBTYPE | VIEWBOUND | SUPERTYPE | HASH | // todo: add LBRACKET
            RPAREN | RBRACKET | RBRACE =>
         false
       case _ =>
@@ -942,6 +935,8 @@ trait Scanners requires SyntaxAnalyzer {
         "';'"
       case NEWLINE =>
         "';'"
+      case NEWLINES =>
+        "';'"
       case COMMA =>
         "','"
       case CASECLASS =>
@@ -980,6 +975,8 @@ trait Scanners requires SyntaxAnalyzer {
         ";"
       case NEWLINE =>
         ";"
+      case NEWLINES =>
+        ";;"
       case COMMA =>
         ","
       case _ =>
