@@ -3,12 +3,12 @@
  * @author  Martin Odersky
  */
 
-// $Id: GenICode.scala 9604 2007-01-14 01:33:08Z emir $
+// $Id: GenICode.scala 10036 2007-02-19 13:56:55Z odersky $
 
 package scala.tools.nsc.backend.icode
 
 import compat.StringBuilder
-import scala.collection.mutable.{Map, HashMap}
+import scala.collection.mutable.{Map, HashMap, ListBuffer, Buffer, Set, HashSet}
 import scala.tools.nsc.symtab._
 
 
@@ -54,6 +54,7 @@ abstract class GenICode extends SubComponent  {
 
     override def apply(unit: CompilationUnit): Unit = {
       this.unit = unit
+      unit.icode.clear
       log("Generating icode for " + unit)
       gen(unit.body)
       this.unit = null
@@ -77,11 +78,12 @@ abstract class GenICode extends SubComponent  {
       case PackageDef(name, stats) =>
         gen(stats, ctx setPackage name)
 
-      case ClassDef(mods, name, tparams, tpt, impl) =>
+      case ClassDef(mods, name, _, _, impl) =>
         log("Generating class: " + tree.symbol.fullNameString)
         ctx setClass (new IClass(tree.symbol) setCompilationUnit unit)
         addClassFields(ctx, tree.symbol);
         classes += tree.symbol -> ctx.clazz
+        unit.icode += ctx.clazz
         gen(impl, ctx)
         ctx setClass null
 
@@ -113,6 +115,8 @@ abstract class GenICode extends SubComponent  {
           rhs match {
             case Block(_, Return(_)) => ()
             case Return(_) => ()
+            case EmptyTree => 
+              error("Concrete method has no definition: " + tree)
             case _ => if (ctx1.bb.isEmpty)
               ctx1.bb.emit(RETURN(m.returnType), rhs.pos)
             else
@@ -150,32 +154,32 @@ abstract class GenICode extends SubComponent  {
      * @return a new context. This is necessary for control flow instructions
      *         which may change the current basic block.
      */
-    private def genStat(tree: Tree, ctx: Context): Context = tree match {
-      case Assign(lhs @ Select(_, _), rhs) =>
-        if (isStaticSymbol(lhs.symbol)) {
+    private def genStat(tree: Tree, ctx: Context): Context = {
+
+      tree match {
+        case Assign(lhs @ Select(_, _), rhs) =>
+          if (isStaticSymbol(lhs.symbol)) {
+            val ctx1 = genLoad(rhs, ctx, toTypeKind(lhs.symbol.info))
+            ctx1.bb.emit(STORE_FIELD(lhs.symbol, true), tree.pos)
+            ctx1
+          } else {
+            var ctx1 = genLoadQualifier(lhs, ctx)
+            ctx1 = genLoad(rhs, ctx1, toTypeKind(lhs.symbol.info))
+            ctx1.bb.emit(STORE_FIELD(lhs.symbol, false), tree.pos)
+            ctx1
+          }
+
+        case Assign(lhs, rhs) =>
           val ctx1 = genLoad(rhs, ctx, toTypeKind(lhs.symbol.info))
-          ctx1.bb.emit(STORE_FIELD(lhs.symbol, true), tree.pos)
+          val Some(l) = ctx.method.lookupLocal(lhs.symbol)
+          ctx1.bb.emit(STORE_LOCAL(l), tree.pos)
           ctx1
-        } else {
-          var ctx1 = genLoadQualifier(lhs, ctx)
-          ctx1 = genLoad(rhs, ctx1, toTypeKind(lhs.symbol.info))
-          ctx1.bb.emit(STORE_FIELD(lhs.symbol, false), tree.pos)
-          ctx1
-        }
 
-      case Assign(lhs, rhs) =>
-//          assert(ctx.method.locals.contains(lhs.symbol) |
-//                 ctx.clazz.fields.contains(lhs.symbol), 
-//                 "Assignment to inexistent local or field: " + lhs.symbol)
-        val ctx1 = genLoad(rhs, ctx, toTypeKind(lhs.symbol.info))
-        val Some(l) = ctx.method.lookupLocal(lhs.symbol)
-        ctx1.bb.emit(STORE_LOCAL(l), tree.pos)
-        ctx1
-
-      case _ =>
-        if (settings.debug.value)
-          log("Passing " + tree + " to genLoad")
-        genLoad(tree, ctx, UNIT)
+        case _ =>
+          if (settings.debug.value)
+            log("Passing " + tree + " to genLoad")
+          genLoad(tree, ctx, UNIT)
+      }
     }
 
     /**
@@ -400,6 +404,8 @@ abstract class GenICode extends SubComponent  {
           val sym = tree.symbol
           var local = new Local(sym, toTypeKind(sym.info), false)
           local = ctx.method.addLocal(local)
+          ctx.scope.add(local)
+          ctx.bb.emit(SCOPE_ENTER(local))
 
           if (rhs == EmptyTree) {
             if (settings.debug.value)
@@ -455,13 +461,19 @@ abstract class GenICode extends SubComponent  {
         case Return(expr) =>
           val returnedKind = toTypeKind(expr.tpe)
           var ctx1 = genLoad(expr, ctx, returnedKind)
+          val oldcleanups = ctx1.cleanups
           for (val op <- ctx1.cleanups) op match {
             case MonitorRelease(m) =>
               ctx1.bb.emit(LOAD_LOCAL(m))
               ctx1.bb.emit(MONITOR_EXIT())
             case Finalizer(f) =>
-              ctx1 = genLoad(f, ctx1, UNIT)
+              if (settings.debug.value) log("removing " + f + " from cleanups: " + ctx1.cleanups)
+              // we have to run this without the same finalizer in
+              // the list, otherwise infinite recursion happens for
+              // finalizers that contain 'return'
+              ctx1 = genLoad(f, ctx1.removeFinalizer(f), UNIT)
           }
+          ctx1.cleanups = oldcleanups
 
           ctx1.bb.emit(RETURN(returnedKind), tree.pos)
           ctx1.bb.enterIgnoreMode
@@ -472,14 +484,14 @@ abstract class GenICode extends SubComponent  {
           val kind = toTypeKind(tree.tpe)
           var handlers = for (val CaseDef(pat, _, body) <- catches.reverse)
             yield pat match {
-              case Typed(Ident(nme.WILDCARD), tpt) => Pair(tpt.tpe.symbol, {
+              case Typed(Ident(nme.WILDCARD), tpt) => Triple(tpt.tpe.symbol, kind, {
                 ctx: Context =>
                   ctx.bb.emit(DROP(REFERENCE(tpt.tpe.symbol)));
                   val ctx1 = genLoad(body, ctx, kind);
                   genLoad(finalizer, ctx1, UNIT);
                 })
 
-              case Ident(nme.WILDCARD) => Pair(definitions.ThrowableClass, {
+              case Ident(nme.WILDCARD) => Triple(definitions.ThrowableClass, kind, {
                 ctx: Context =>
                   ctx.bb.emit(DROP(REFERENCE(definitions.ThrowableClass)))
                   val ctx1 = genLoad(body, ctx, kind)
@@ -490,29 +502,30 @@ abstract class GenICode extends SubComponent  {
                 val exception = new Local(pat.symbol, toTypeKind(pat.symbol.tpe), false)
                 ctx.method.addLocal(exception);
 
-                Pair(pat.symbol.tpe.symbol, {
-                     ctx: Context => 
-                       ctx.bb.emit(STORE_LOCAL(exception), pat.pos);
-                       val ctx1 = genLoad(body, ctx, kind);
-                       genLoad(finalizer, ctx1, UNIT);
-                     })
+                Triple(pat.symbol.tpe.symbol, kind, {
+                  ctx: Context =>
+                    ctx.bb.emit(STORE_LOCAL(exception), pat.pos);
+                  val ctx1 = genLoad(body, ctx, kind);
+                  genLoad(finalizer, ctx1, UNIT);
+                })
             }
 
-          if (finalizer != EmptyTree) 
-            handlers = Pair(NoSymbol, {
-                            ctx: Context => 
-                              val exception = new Local(ctx.method.symbol.newVariable(finalizer.pos, unit.fresh.newName("exc"))
-                                    .setFlag(Flags.SYNTHETIC)
-                                    .setInfo(definitions.ThrowableClass.tpe),
-                                  REFERENCE(definitions.ThrowableClass), false);
-                              ctx.method.addLocal(exception);
-                              ctx.bb.emit(STORE_LOCAL(exception));
-                              val ctx1 = genLoad(finalizer, ctx, UNIT);
-                              ctx1.bb.emit(LOAD_LOCAL(exception));
-                              ctx1.bb.emit(THROW());
-                              ctx1.bb.enterIgnoreMode;
-                              ctx1
-                            }) :: handlers;
+          if (finalizer != EmptyTree)
+            handlers = Triple(NoSymbol, kind, {
+              ctx: Context =>
+                val exception = new Local(ctx.method.symbol
+                                          .newVariable(finalizer.pos, unit.fresh.newName("exc"))
+                                          .setFlag(Flags.SYNTHETIC)
+                                          .setInfo(definitions.ThrowableClass.tpe),
+                                          REFERENCE(definitions.ThrowableClass), false);
+              ctx.method.addLocal(exception);
+              ctx.bb.emit(STORE_LOCAL(exception));
+              val ctx1 = genLoad(finalizer, ctx, UNIT);
+              ctx1.bb.emit(LOAD_LOCAL(exception));
+              ctx1.bb.emit(THROW());
+              ctx1.bb.enterIgnoreMode;
+              ctx1
+            }) :: handlers;
 
           ctx.Try(
             bodyCtx => {
@@ -678,10 +691,10 @@ abstract class GenICode extends SubComponent  {
 
             ctx1.bb.close
             ctx1.newBlock
-          } else if (isPrimitive(fun.symbol)) { // primitive method call
+          } else if (isPrimitive(sym)) { // primitive method call
             val Select(receiver, _) = fun
 
-            val code = scalaPrimitives.getPrimitive(fun.symbol, receiver.tpe)
+            val code = scalaPrimitives.getPrimitive(sym, receiver.tpe)
             var ctx1 = ctx
 
             if (scalaPrimitives.isArithmeticOp(code)) {
@@ -725,6 +738,7 @@ abstract class GenICode extends SubComponent  {
 
               if (settings.debug.value)
                 log("synchronized block start");
+
               ctx1 = ctx1.Try(
                 bodyCtx => {
                   val ctx1 = genLoad(args.head, bodyCtx, expectedType /* toTypeKind(tree.tpe.resultType) */)
@@ -732,7 +746,8 @@ abstract class GenICode extends SubComponent  {
                   ctx1.bb.emit(MONITOR_EXIT(), tree.pos)
                   ctx1
                 }, List(
-                Pair(NoSymbol, exhCtx => {
+		  // tree.tpe / fun.tpe is object, which is no longer true after this transformation
+                  Triple(NoSymbol, expectedType, exhCtx => {
                   exhCtx.bb.emit(LOAD_LOCAL(monitor))
                   exhCtx.bb.emit(MONITOR_EXIT(), tree.pos)
                   exhCtx.bb.emit(THROW())
@@ -748,9 +763,8 @@ abstract class GenICode extends SubComponent  {
               genCoercion(tree, ctx1, code)
               generatedType = scalaPrimitives.generatedKind(code)
             } else
-              abort("Primitive operation not handled yet: " + 
-                    fun.symbol.fullNameString + "(" + fun.symbol.simpleName + ") "
-                    + " at: " + unit.position(tree.pos));
+              abort("Primitive operation not handled yet: " + sym.fullNameString + "(" +
+                    fun.symbol.simpleName + ") " + " at: " + unit.position(tree.pos));
             ctx1
           } else {  // normal method call
             if (settings.debug.value)
@@ -767,7 +781,7 @@ abstract class GenICode extends SubComponent  {
               if (invokeStyle.hasInstance) genLoadQualifier(fun, ctx)
               else ctx
 
-            ctx1 = genLoadArguments(args, fun.symbol.info.paramTypes, ctx1)
+            ctx1 = genLoadArguments(args, sym.info.paramTypes, ctx1)
 
             val hostClass = fun match {
               case Select(qualifier, _)
@@ -855,9 +869,12 @@ abstract class GenICode extends SubComponent  {
           ctx
 
         case Block(stats, expr) =>
-          assert(!(ctx.method eq null), "Block outside method")
-          val ctx1 = genStat(stats, ctx)
-          genLoad(expr, ctx1, expectedType)
+//          assert(!(ctx.method eq null), "Block outside method")
+          ctx.enterScope
+          var ctx1 = genStat(stats, ctx)
+          ctx1 = genLoad(expr, ctx1, expectedType)
+          ctx1.exitScope
+          ctx1
 
         case Typed(expr, _) =>
           genLoad(expr, ctx, expectedType)
@@ -975,6 +992,11 @@ abstract class GenICode extends SubComponent  {
     def isStaticSymbol(s: Symbol): Boolean =
       s.hasFlag(Flags.STATIC) || s.hasFlag(Flags.STATICMEMBER) || s.owner.isImplClass
 
+//     def isUnbox(s: Symbol): Boolean = (
+//       s.name.==(nme.unbox) && s.owner.isModuleClass
+//       && s.owner.linkedClassOfClass.isSubClass(definitions.AnyValClass)
+//     )
+
     /**
      * Generate code that loads args into label parameters.
      */
@@ -1036,7 +1058,7 @@ abstract class GenICode extends SubComponent  {
      *  @param ctx  ...
      *  @param cast ...
      */
-    def genBoxedConversion(from: TypeKind, to: TypeKind, ctx: Context, cast: Boolean) = {
+    def genBoxedConversion(from: TypeKind, to: TypeKind, ctx: Context, cast: Boolean): Unit = {
       assert(to.isValueType || to.isArrayType,
              "Expecting conversion to value type: " + to)
 
@@ -1046,7 +1068,12 @@ abstract class GenICode extends SubComponent  {
         case ARRAY(elem) =>
           definitions.boxedArrayClass(elem.toType.symbol)
         case _ =>
-          definitions.boxedClass(to.toType.symbol)
+          if (cast) {
+            ctx.bb.emit(UNBOX(to))
+            return
+          }
+          else
+            definitions.boxedClass(to.toType.symbol)
       }
 
       if (cast) {
@@ -1361,6 +1388,8 @@ abstract class GenICode extends SubComponent  {
           eqEqTempVar.setInfo(definitions.AnyRefClass.typeConstructor)
           val local = new Local(eqEqTempVar, REFERENCE(definitions.AnyRefClass), false)
           ctx.method.addLocal(local)
+          local.start = unit.position(l.pos).line
+          local.end   = unit.position(r.pos).line
           local
       }
 
@@ -1433,8 +1462,12 @@ abstract class GenICode extends SubComponent  {
         case Nil => ()
 
         case vparams :: Nil =>
-          for (val p <- vparams)
-            ctx.method.addParam(new Local(p.symbol, toTypeKind(p.symbol.info), true));
+          for (val p <- vparams) {
+            val lv = new Local(p.symbol, toTypeKind(p.symbol.info), true)
+            ctx.method.addParam(lv)
+            ctx.scope.add(lv)
+            ctx.bb.varsInScope += lv
+          }
           ctx.method.params = ctx.method.params.reverse
 
         case _ =>
@@ -1603,6 +1636,9 @@ abstract class GenICode extends SubComponent  {
       /** The current exception handler, when we generate code for one. */
       var currentExceptionHandler: Option[ExceptionHandler] = None
 
+      /** The current local variable scope. */
+      var scope: Scope = EmptyScope
+
       var handlerCount = 0
 
       override def toString(): String = {
@@ -1614,6 +1650,7 @@ abstract class GenICode extends SubComponent  {
         buf.append("\tlabels: ").append(labels).append('\n')
         buf.append("\texception handlers: ").append(handlers).append('\n')
         buf.append("\tcleanups: ").append(cleanups).append('\n')
+        buf.append("\tscope: ").append(scope).append('\n')
         buf.toString()
       }
 
@@ -1629,6 +1666,7 @@ abstract class GenICode extends SubComponent  {
         this.handlerCount = other.handlerCount
         this.cleanups = other.cleanups
         this.currentExceptionHandler = other.currentExceptionHandler
+        this.scope = other.scope
       }
 
       def setPackage(p: Name): this.type = {
@@ -1670,7 +1708,7 @@ abstract class GenICode extends SubComponent  {
 
       def removeFinalizer(f: Tree): this.type = {
         assert(cleanups.head == f,
-               "Illegal nesting of cleanup operations: " + cleanups + " while exiting finalizer" + f);
+               "Illegal nesting of cleanup operations: " + cleanups + " while exiting finalizer " + f);
         cleanups = cleanups.tail
         this
       }
@@ -1682,6 +1720,8 @@ abstract class GenICode extends SubComponent  {
         ctx1.method.code = new Code(m.symbol.simpleName.toString())
         ctx1.bb = ctx1.method.code.startBlock
         ctx1.defdef = d
+        ctx1.scope = EmptyScope
+        ctx1.enterScope
         ctx1
       }
 
@@ -1693,17 +1733,31 @@ abstract class GenICode extends SubComponent  {
           case Some(e) => e.addBlock(block)
           case None    => ()
         }
+        block.varsInScope = new HashSet()
+        block.varsInScope ++= scope.varsInScope
         new Context(this) setBasicBlock block;
       }
 
+      def enterScope = {
+        scope = new Scope(scope)
+      }
+
+      def exitScope = {
+        if (bb.size > 0) {
+          scope.locals foreach { lv => bb.emit(SCOPE_EXIT(lv)) }
+        }
+        scope = scope.outer
+      }
+
       /** Create a new exception handler and adds it in the list
        * of current exception handlers. All new blocks will be
        * 'covered' by this exception handler (in addition to the 
        * previously active handlers).
        */
-      def newHandler(cls: Symbol): ExceptionHandler = {
+      def newHandler(cls: Symbol, resultKind: TypeKind): ExceptionHandler = {
         handlerCount = handlerCount + 1
         val exh = new ExceptionHandler(method, "" + handlerCount, cls)
+	exh.resultKind = resultKind
         method.addHandler(exh)
         handlers = exh :: handlers
         if (settings.debug.value)
@@ -1754,14 +1808,14 @@ abstract class GenICode extends SubComponent  {
        *   } ))</code>
        */
       def Try(body: Context => Context, 
-              handlers: List[Pair[Symbol, (Context => Context)]],
+              handlers: List[Triple[Symbol, TypeKind, (Context => Context)]],
               finalizer: Tree) = {
         val outerCtx = this.dup
         val afterCtx = outerCtx.newBlock
 
         val exhs = handlers.map { handler =>
-            val exh = this.newHandler(handler._1)
-            val ctx1 = handler._2(outerCtx.enterHandler(exh))
+            val exh = this.newHandler(handler._1, handler._2)
+            val ctx1 = handler._3(outerCtx.enterHandler(exh))
             ctx1.bb.emit(JUMP(afterCtx.bb))
             ctx1.bb.close
             exh
@@ -1910,4 +1964,25 @@ abstract class GenICode extends SubComponent  {
         failure.addCallingInstruction(this)
     }
 
+  /** Local variable scopes. Keep track of line numbers for debugging info. */
+  class Scope(val outer: Scope) {
+    val locals: ListBuffer[Local] = new ListBuffer
+
+    def add(l: Local) =
+      locals += l
+    
+    def remove(l: Local) =
+      locals -= l
+      
+    /** Return all locals that are in scope. */
+    def varsInScope: Buffer[Local] = outer.varsInScope ++ locals
+    
+    override def toString() = 
+      outer.toString() + locals.mkString("[", ", ", "]")
+  }
+
+  object EmptyScope extends Scope(null) {
+    override def toString() = "[]"
+    override def varsInScope: Buffer[Local] = new ListBuffer
+  }
 }
