@@ -3,7 +3,7 @@
  * @author  Iulian Dragos
  */
 
-// $Id: GenJVM.scala 9546 2007-01-03 15:56:13Z odersky $
+// $Id: GenJVM.scala 9980 2007-02-13 22:06:58Z odersky $
 
 package scala.tools.nsc.backend.jvm
 
@@ -11,7 +11,7 @@ import java.io.File
 import java.nio.ByteBuffer
 
 import scala.collection.immutable.{Set, ListSet}
-import scala.collection.mutable.{Map, HashMap}
+import scala.collection.mutable.{Map, HashMap, HashSet}
 import scala.tools.nsc.symtab._
 import scala.tools.nsc.util.Position
 
@@ -37,7 +37,7 @@ abstract class GenJVM extends SubComponent {
   class JvmPhase(prev: Phase) extends StdPhase(prev) {
 
     override def erasedTypes = true
-    val codeGenerator = new BytecodeGenerator
+    object codeGenerator extends BytecodeGenerator
 
     override def run: Unit = {
       if (settings.debug.value) inform("[running phase " + name + " on icode]")
@@ -62,14 +62,17 @@ abstract class GenJVM extends SubComponent {
     val stringBufferType = new JObjectType(JAVA_LANG_STRINGBUFFER)
     val toStringType = new JMethodType(JObjectType.JAVA_LANG_STRING, JType.EMPTY_ARRAY)
 
+    def attributeType(name: String) = 
+      atPhase(currentRun.typerPhase)(definitions.getClass(name).tpe)
+
     // Scala attributes
-    val SerializableAttr = definitions.SerializableAttr.tpe
-    val SerialVersionUID = definitions.getClass("scala.SerialVersionUID").tpe
-    val CloneableAttr    = definitions.getClass("scala.cloneable").tpe
-    val TransientAtt     = definitions.getClass("scala.transient").tpe
-    val VolatileAttr     = definitions.getClass("scala.volatile").tpe
-    val RemoteAttr       = definitions.getClass("scala.remote").tpe
-    val ThrowsAttr       = definitions.getClass("scala.throws").tpe
+    val SerializableAttr = atPhase(currentRun.typerPhase)(definitions.SerializableAttr.tpe)
+    val SerialVersionUID = attributeType("scala.SerialVersionUID")
+    val CloneableAttr    = attributeType("scala.cloneable")
+    val TransientAtt     = attributeType("scala.transient")
+    val VolatileAttr     = attributeType("scala.volatile")
+    val RemoteAttr       = attributeType("scala.remote")
+    val ThrowsAttr       = attributeType("scala.throws")
 
     val CloneableClass   =
       if (forCLDC) null else definitions.getClass("java.lang.Cloneable")
@@ -85,7 +88,7 @@ abstract class GenJVM extends SubComponent {
     var jmethod: JMethod = _
     var jcode: JExtendedCode = _
 
-    var innerClasses: Set[Symbol] = ListSet.Empty // referenced inner classes
+    var innerClasses: Set[Symbol] = ListSet.empty // referenced inner classes
 
     val fjbgContext =
       if (settings.target.value == "jvm-1.5") new FJBGContext(49, 0)
@@ -122,9 +125,7 @@ abstract class GenJVM extends SubComponent {
       val outfile = getFile(jclass, ".class")
       jclass.writeTo(outfile)
       val file = scala.tools.nsc.io.AbstractFile.getFile(outfile)
-      informProgress("wrote " + outfile + " " +
-                     (if (file ne null) "" + file.file + " " + file.file.exists()
-                      else "no file"))
+      informProgress("wrote " + outfile)
     }
 
     var serialVUID: Option[Long] = None
@@ -135,7 +136,7 @@ abstract class GenJVM extends SubComponent {
         log("Generating class " + c.symbol +
             " flags: " + Flags.flagsToString(c.symbol.flags))
       clasz = c
-      innerClasses = ListSet.Empty
+      innerClasses = ListSet.empty
 
       var parents = c.symbol.info.parents
       var ifaces  = JClass.NO_INTERFACES
@@ -148,13 +149,13 @@ abstract class GenJVM extends SubComponent {
 
       if (!forCLDC)
         for (val attr <- c.symbol.attributes) attr match {
-          case Triple(SerializableAttr, _, _) =>
+          case AttrInfo(SerializableAttr, _, _) =>
             parents = parents ::: List(definitions.SerializableClass.tpe)
-          case Triple(CloneableAttr, _, _)  =>
+          case AttrInfo(CloneableAttr, _, _)  =>
             parents = parents ::: List(CloneableClass.tpe)
-          case Triple(SerialVersionUID, value :: _, _) =>
+          case AttrInfo(SerialVersionUID, value :: _, _) =>
             serialVUID = Some(value.longValue)
-          case Triple(RemoteAttr, _, _) =>
+          case AttrInfo(RemoteAttr, _, _) =>
             parents = parents ::: List(RemoteInterface.tpe)
             remoteClass = true
           case _ => ()
@@ -203,7 +204,7 @@ abstract class GenJVM extends SubComponent {
 
     def addExceptionsAttribute(sym: Symbol): Unit = {
       val Pair(excs, others) = sym.attributes.partition((a => a match {
-        case Triple(ThrowsAttr, _, _) => true
+        case AttrInfo(ThrowsAttr, _, _) => true
         case _ => false
       }))
       if (excs isEmpty) return;
@@ -216,8 +217,8 @@ abstract class GenJVM extends SubComponent {
       // put some radom value; the actual number is determined at the end
       buf.putShort(0xbaba.toShort)
 
-      for (val Triple(ThrowsAttr, List(exc), _) <- excs.removeDuplicates) {
-        buf.putShort(cpool.addClass(exc.typeValue.toString()).shortValue)
+      for (val AttrInfo(ThrowsAttr, List(exc), _) <- excs.removeDuplicates) {
+        buf.putShort(cpool.addClass(javaName(exc.typeValue.symbol)).shortValue)
         nattr = nattr + 1
       }
 
@@ -277,7 +278,8 @@ abstract class GenJVM extends SubComponent {
       // put some radom value; the actual number of annotations is determined at the end
       buf.putShort(0xbaba.toShort)
 
-      for (val Triple(typ, consts, nvPairs) <- attributes; typ.symbol.hasFlag(Flags.JAVA)) {
+      for (val AttrInfo(typ, consts, nvPairs) <- attributes; 
+           typ.symbol isNonBottomSubClass definitions.ClassfileAnnotationClass) {
         nattr = nattr + 1
         val jtype = javaType(typ)
         buf.putShort(cpool.addUtf8(jtype.getSignature()).toShort)
@@ -310,8 +312,8 @@ abstract class GenJVM extends SubComponent {
 
     def addParamAnnotations(pattrss: List[List[AttrInfo]]): Unit = {
       val attributes = for (val attrs <- pattrss) yield
-        for (val attr @ Triple(tpe, _, _) <- attrs;
-             tpe.symbol hasFlag Flags.JAVA) yield attr;
+        for (val attr @ AttrInfo(tpe, _, _) <- attrs;
+             tpe.symbol isNonBottomSubClass definitions.ClassfileAnnotationClass) yield attr;
       if (attributes.forall(.isEmpty)) return;
 
       val buf: ByteBuffer = ByteBuffer.allocate(2048)
@@ -369,9 +371,9 @@ abstract class GenJVM extends SubComponent {
       var attributes = 0
 
       f.symbol.attributes foreach { a => a match {
-        case Triple(TransientAtt, _, _) =>
+        case AttrInfo(TransientAtt, _, _) =>
           attributes = attributes | JAccessFlags.ACC_TRANSIENT
-        case Triple(VolatileAttr, _, _) =>
+        case AttrInfo(VolatileAttr, _, _) =>
           attributes = attributes | JAccessFlags.ACC_VOLATILE
         case _ => ();
       }}
@@ -410,11 +412,11 @@ abstract class GenJVM extends SubComponent {
         jmethod.addAttribute(fjbgContext.JOtherAttribute(jclass, jmethod, "Bridge",
                                                          new Array[Byte](0)))
       if ((remoteClass ||
-          (m.symbol.attributes contains Triple(RemoteAttr, Nil, Nil))) &&
+          (m.symbol.attributes contains AttrInfo(RemoteAttr, Nil, Nil))) &&
           jmethod.isPublic() && !forCLDC)
         {
           m.symbol.attributes =
-            Triple(ThrowsAttr, List(Constant(RemoteException)), List()) :: m.symbol.attributes;
+            AttrInfo(ThrowsAttr, List(Constant(RemoteException)), List()) :: m.symbol.attributes;
         }
 
       if (!jmethod.isAbstract()) {
@@ -635,6 +637,9 @@ abstract class GenJVM extends SubComponent {
       });
     }
 
+    /** local variables whose scope appears in this block. */
+    var varsInBlock: collection.mutable.Set[Local] = new HashSet
+
     def genBlock(b: BasicBlock): Unit = {
       labels(b).anchorToNext()
 
@@ -643,6 +648,7 @@ abstract class GenJVM extends SubComponent {
       var lastMappedPC = 0
       var lastLineNr = 0
       var crtPC = 0
+      varsInBlock.clear
 
       b traverse ( instr => {
         class CompilationError(msg: String) extends Error {
@@ -657,9 +663,6 @@ abstract class GenJVM extends SubComponent {
         }
         def assert(cond: Boolean, msg: String) = if (!cond) throw new CompilationError(msg);
 
-        if (b.lastInstruction == instr)
-          endPC(b) = jcode.getPC();
-
         instr match {
           case THIS(clasz) =>
             jcode.emitALOAD_0()
@@ -978,7 +981,21 @@ abstract class GenJVM extends SubComponent {
             jcode.emitMONITORENTER()
 
           case MONITOR_EXIT() =>
-            jcode.emitMONITOREXIT()
+            jcode.emitMONITOREXIT()            
+
+          case SCOPE_ENTER(lv) => 
+            varsInBlock += lv
+            lv.start = jcode.getPC()
+            
+          case SCOPE_EXIT(lv) =>
+            if (varsInBlock contains lv) {
+              lv.ranges = Pair(lv.start, jcode.getPC()) :: lv.ranges
+              varsInBlock -= lv
+            } else if (b.varsInScope contains lv) {
+              lv.ranges = Pair(labels(b).getAnchor(), jcode.getPC()) :: lv.ranges
+              b.varsInScope -= lv
+            } else
+              assert(false, "Illegal local var nesting: " + method)
         }
 
         crtPC = jcode.getPC()
@@ -989,6 +1006,9 @@ abstract class GenJVM extends SubComponent {
             log("Warning: wrong position in: " + method)
             lastLineNr
         }
+        if (b.lastInstruction == instr)
+          endPC(b) = jcode.getPC();
+
 	//System.err.println("CRTLINE: " + instr.pos + " " + 
 	//	   /* (if (instr.pos < clasz.cunit.source.content.length) clasz.cunit.source.content(instr.pos) else '*') + */ " " + crtLine);
 
@@ -998,7 +1018,15 @@ abstract class GenJVM extends SubComponent {
           lastLineNr   = crtLine
         }
 
-      });
+      }); // b.traverse 
+      
+      // local vars that survived this basic block 
+      for (val lv <- varsInBlock) {
+        lv.ranges = Pair(lv.start, jcode.getPC()) :: lv.ranges
+      }
+      for (val lv <- b.varsInScope) {
+        lv.ranges = Pair(labels(b).getAnchor(), jcode.getPC()) :: lv.ranges
+      }
     }
 
     /**
@@ -1174,6 +1202,7 @@ abstract class GenJVM extends SubComponent {
       }
     }
 
+    /** For each basic block, the first PC address following it. */
     val endPC: HashMap[BasicBlock, Int] = new HashMap()
     val labels: HashMap[BasicBlock, JCode$Label] = new HashMap()
     val conds: HashMap[TestOp, Int] = new HashMap()
@@ -1332,6 +1361,7 @@ abstract class GenJVM extends SubComponent {
       jf = jf | (if (sym hasFlag Flags.INTERFACE) ACC_INTERFACE else 0)
       jf = jf | (if ((sym hasFlag Flags.FINAL) && !sym.enclClass.hasFlag(Flags.INTERFACE)) ACC_FINAL else 0)
       jf = jf | (if (isStaticSymbol(sym)) ACC_STATIC else 0)
+      jf = jf | (if (sym hasFlag Flags.SYNTHETIC) ACC_SYNTHETIC else 0)
       jf
     }
 
@@ -1342,7 +1372,7 @@ abstract class GenJVM extends SubComponent {
     def needsInterfaceCall(sym: Symbol): Boolean =
       sym.hasFlag(Flags.INTERFACE) ||
       (sym.hasFlag(Flags.JAVA) &&
-       sym.isNonBottomSubClass(definitions.AttributeClass))
+       sym.isNonBottomSubClass(definitions.ClassfileAnnotationClass))
 
 
     def javaType(t: TypeKind): JType = t match {
@@ -1392,21 +1422,23 @@ abstract class GenJVM extends SubComponent {
         val pool = jclass.getConstantPool()
         val pc = jcode.getPC()
         var anonCounter = 0
-        val locals = if (jmethod.isStatic()) vars.length else 1 + vars.length
+        var entries = 0
+        vars.foreach { lv => lv.ranges = mergeEntries(lv.ranges.reverse); entries = entries + lv.ranges.length }
+        if (!jmethod.isStatic()) entries = entries + 1
 
-        val lvTab = ByteBuffer.allocate(2 + 10 * locals)
-        def emitEntry(name: String, signature: String, idx: Short): Unit = {
-          lvTab.putShort(0.asInstanceOf[Short])
-          lvTab.putShort(pc.asInstanceOf[Short])
+        val lvTab = ByteBuffer.allocate(2 + 10 * entries)
+        def emitEntry(name: String, signature: String, idx: Short, start: Short, end: Short): Unit = {
+          lvTab.putShort(start)
+          lvTab.putShort(end)
           lvTab.putShort(pool.addUtf8(name).asInstanceOf[Short])
           lvTab.putShort(pool.addUtf8(signature).asInstanceOf[Short])
           lvTab.putShort(idx)
         }
 
-        lvTab.putShort(locals.asInstanceOf[Short])
+        lvTab.putShort(entries.asInstanceOf[Short])
 
         if (!jmethod.isStatic()) {
-          emitEntry("this", jclass.getType().getSignature(), 0)
+          emitEntry("this", jclass.getType().getSignature(), 0, 0.asInstanceOf[Short], pc.asInstanceOf[Short])
         }
 
         for (val lv <- vars) {
@@ -1415,8 +1447,11 @@ abstract class GenJVM extends SubComponent {
               "<anon" + anonCounter + ">"
             } else javaName(lv.sym)
 
-            emitEntry(name, javaType(lv.kind).getSignature(),
-                      indexOf(lv).asInstanceOf[Short])
+            val index = indexOf(lv).asInstanceOf[Short]
+            val tpe   = javaType(lv.kind).getSignature()
+            for (val Pair(start, end) <- lv.ranges) {
+              emitEntry(name, tpe, index, start.asInstanceOf[Short], (end - start).asInstanceOf[Short])
+            }
         }
         val attr =
             fjbgContext.JOtherAttribute(jclass,
@@ -1426,6 +1461,15 @@ abstract class GenJVM extends SubComponent {
         jcode.addAttribute(attr)
     }
 
+    
+    /** Merge adjacent ranges. */
+    private def mergeEntries(ranges: List[Pair[Int, Int]]): List[Pair[Int, Int]] = 
+      (ranges.foldLeft(Nil: List[Pair[Int, Int]]) { (collapsed: List[Pair[Int, Int]], p: Pair[Int, Int]) => Pair(collapsed, p) match {
+        case Pair(Nil, _) => List(p)
+        case Pair(Pair(s1, e1) :: rest, Pair(s2, e2)) if (e1 == s2) => Pair(s1, e2) :: rest
+        case _ => p :: collapsed
+      }}).reverse
+
     def assert(cond: Boolean, msg: String) = if (!cond) {
       dump(method)
       throw new Error(msg + "\nMethod: " + method)
