@@ -3,7 +3,7 @@
  * @author  Iulian Dragos
  */
 
-// $Id: ClosureElimination.scala 9388 2006-11-27 15:04:01Z mihaylov $
+// $Id: ClosureElimination.scala 9967 2007-02-12 15:29:23Z dragos $
 
 package scala.tools.nsc.backend.opt;
 
@@ -25,17 +25,13 @@ abstract class ClosureElimination extends SubComponent {
 
   /** The Inlining phase.
    */
-  class ClosureEliminationPhase(prev: Phase) extends StdPhase(prev) {
+  class ClosureEliminationPhase(prev: Phase) extends ICodePhase(prev) {
 
-    override def erasedTypes = true;
+    def name = phaseName
     val closser = new ClosureElim;
 
-    override def run: Unit = {
-      if (settings.debug.value) inform("[running phase " + name + " on icode]");
-      classes.values foreach closser.analyzeClass;
-    }
-    override def apply(unit: CompilationUnit): Unit =
-      abort("Inlining works on icode classes, not on compilation units!");
+    override def apply(c: IClass): Unit =
+      closser.analyzeClass(c)
   }
 
   /** 
