@@ -3,7 +3,7 @@
  * @author  Iulian Dragos
  */
 
-// $Id: Inliners.scala 9388 2006-11-27 15:04:01Z mihaylov $
+// $Id: Inliners.scala 9967 2007-02-12 15:29:23Z dragos $
 
 package scala.tools.nsc.backend.opt;
 
@@ -25,17 +25,14 @@ abstract class Inliners extends SubComponent {
 
   /** The Inlining phase.
    */
-  class InliningPhase(prev: Phase) extends StdPhase(prev) {
+  class InliningPhase(prev: Phase) extends ICodePhase(prev) {
 
-    override def erasedTypes = true;
+    def name = phaseName
     val inliner = new Inliner;
 
-    override def run: Unit = {
-      if (settings.debug.value) inform("[running phase " + name + " on icode]");
-      classes.values foreach inliner.analyzeClass;
-    }
-    override def apply(unit: CompilationUnit): Unit =
-      abort("Inlining works on icode classes, not on compilation units!");
+    override def apply(c: IClass): Unit =
+      inliner.analyzeClass(c)
+
   }
 
   /** 
@@ -221,7 +218,11 @@ abstract class Inliners extends SubComponent {
        caller.exh = (callee.exh map translateExh) ::: caller.exh;
      }
 
-    val InlineAttr = if (settings.inline.value) global.definitions.getClass("scala.inline").tpe else null;
+    val InlineAttr = if (settings.inline.value) try { 
+      global.definitions.getClass("scala.inline").tpe 
+    } catch {
+      case e: FatalError => null
+    } else null;
 
     def analyzeClass(cls: IClass): Unit = if (settings.inline.value) {
       if (settings.debug.value)
@@ -266,7 +267,7 @@ abstract class Inliners extends SubComponent {
                     if (   classes.contains(receiver) 
                         && (isClosureClass(receiver)
                             || concreteMethod.isFinal
-                            || msym.attributes.exists(a => a._1 == InlineAttr))) {
+                            || msym.attributes.exists(a => a.atp == InlineAttr))) {
                       classes(receiver).lookupMethod(concreteMethod) match {
                         case Some(inc) =>
                           if (inc != m && (inc.code ne null)
