@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Definitions.scala 9454 2006-12-03 00:32:39Z emir $
+// $Id: Definitions.scala 10005 2007-02-16 12:02:06Z odersky $
 
 package scala.tools.nsc.symtab
 
@@ -41,10 +41,17 @@ trait Definitions requires SymbolTable {
     var NullPointerExceptionClass: Symbol = _
     var NonLocalReturnExceptionClass: Symbol = _
 
+    var ValueTypeClass: Symbol = _ // System.ValueType
+    var DelegateClass: Symbol = _ // System.MulticastDelegate
+    var Delegate_scalaCallers: List[Symbol] = List()
+    // Symbol -> (Symbol, Type): scalaCaller -> (scalaMethodSym, DelegateType)
+    // var Delegate_scalaCallerInfos: HashMap[Symbol, Pair[Symbol, Type]] = _
+    var Delegate_scalaCallerTargets: HashMap[Symbol, Symbol] = _
+
     // the scala value classes
     var UnitClass: Symbol = _
     var BooleanClass: Symbol = _
-      def Boolean_not = getMember(BooleanClass, nme.ZNOT)
+      def Boolean_not = getMember(BooleanClass, nme.UNARY_!)
       def Boolean_and = getMember(BooleanClass, nme.ZAND)
       def Boolean_or  = getMember(BooleanClass, nme.ZOR)
     var ByteClass: Symbol = _
@@ -58,7 +65,9 @@ trait Definitions requires SymbolTable {
     // the scala reference classes
     var ScalaObjectClass: Symbol = _
       def ScalaObjectClass_tag = getMember(ScalaObjectClass, nme.tag)
-    var AttributeClass: Symbol = _
+    var AnnotationClass: Symbol = _
+    var ClassfileAnnotationClass: Symbol = _
+    var StaticAnnotationClass: Symbol = _
     //var ChannelClass: Symbol = _
     //  def Channel_send = getMember(ChannelClass, nme.send)
     //  def Channel_receive = getMember(ChannelClass, nme.receive)
@@ -97,8 +106,9 @@ trait Definitions requires SymbolTable {
       def checkDefinedMethod = getMember(ScalaRunTimeModule, "checkDefined")
     var RepeatedParamClass: Symbol = _
     var ByNameParamClass: Symbol = _
+    var UnsealedClass: Symbol = _
 
-    val MaxTupleArity = 9
+    val MaxTupleArity = 22
     val TupleClass: Array[Symbol] = new Array(MaxTupleArity + 1)
       def tupleField(n: Int, j: Int) = getMember(TupleClass(n), "_" + j)
       def isTupleType(tp: Type): Boolean = tp match {
@@ -116,8 +126,10 @@ trait Definitions requires SymbolTable {
     val MaxProductArity = 22
     /* <unapply> */
     val ProductClass: Array[Symbol] = new Array(MaxProductArity + 1)
-      def productProj(n: Int, j: Int) = getMember(ProductClass(n), "_" + j)
-      def isProductType(tp: Type): Boolean = tp match {
+      def productProj(z:Symbol, j: Int): Symbol = getMember(z, nme.Product_(j))
+      def productProj(n: Int,   j: Int): Symbol = productProj(ProductClass(n), j)
+    /** returns true if this type is exactly ProductN[T1,...,Tn], not some subclass */
+      def isExactProductType(tp: Type): Boolean = tp match {
         case TypeRef(_, sym, elems) =>
           elems.length <= MaxProductArity && sym == ProductClass(elems.length);
         case _ =>
@@ -131,14 +143,28 @@ trait Definitions requires SymbolTable {
           typeRef(sym.typeConstructor.prefix, sym, elems)
         } else NoType
 
+    /** if tpe <: ProductN[T1,...,TN], returns Some(T1,...,TN) else None */
+    def getProductArgs(tpe: Type): Option[List[Type]] = 
+      tpe.baseClasses.find { x => definitions.isExactProductType(x.tpe) } match {
+        case Some(p) => Some(tpe.baseType(p).typeArgs)
+	case _       => None
+      }
+
     var OptionClass: Symbol = _
-    def SomeClass: Symbol = getClass("scala.Some")
-    def NoneClass: Symbol = getModule("scala.None")
+
+    private var SomeClass_ : Symbol = null
+    def SomeClass: Symbol = { if(SomeClass_ eq null) SomeClass_ = getClass("scala.Some"); SomeClass_ }
+    private var NoneClass_ : Symbol = null
+    def NoneClass: Symbol = { if(NoneClass_ eq null) SomeClass_ = getModule("scala.None"); NoneClass_ }
 
     def isOptionType(tp: Type) = tp match {
       case TypeRef(_, sym, List(_)) if sym == OptionClass => true
       case _ => false
     }
+    def isOptionOrSomeType(tp: Type) = tp match {
+      case TypeRef(_, sym, List(_)) => sym == OptionClass || sym == SomeClass
+      case _ => false
+    }
     def optionType(tp: Type) =
       typeRef(OptionClass.typeConstructor.prefix, OptionClass, List(tp))
 
@@ -154,21 +180,79 @@ trait Definitions requires SymbolTable {
       case _ => false
     }
 
-    def optionOfProductElems(tp: Type): List[Type] = {
-      assert(tp.symbol == OptionClass)
-      val prod = tp.typeArgs.head
-      if (prod.symbol == UnitClass) List()
-      else prod.baseClasses.find { x => isProductType(x.tpe) } match {
-        case Some(p) => prod.baseType(p).typeArgs
+    def unapplyUnwrap(tpe:Type) = tpe match {
+      case PolyType(_,MethodType(_, res)) => res
+      case MethodType(_, res)             => res
+      case tpe                            => tpe
+    }
+    
+    /** returns type list for return type of the extraction */
+    def unapplyTypeList(ufn: Symbol, ufntpe: Type) = {
+      assert(ufn.isMethod)
+      //Console.println("utl "+ufntpe+" "+ufntpe.symbol)
+      ufn.name match {
+	case nme.unapply    => unapplyTypeListFromReturnType(ufntpe)
+	case nme.unapplySeq => unapplyTypeListFromReturnTypeSeq(ufntpe)
+	case _ => throw new IllegalArgumentException("expected function symbol of extraction")
+      }
+    }
+    /** (the inverse of unapplyReturnTypeSeq)
+     *  for type Boolean, returns Nil
+     *  for type Option[T] or Some[T]:
+     *   - returns T0...Tn if n>0 and T <: Product[T0...Tn]]
+     *   - returns T otherwise
+     */
+    def unapplyTypeListFromReturnType(tp1: Type): List[Type] =  { // rename: unapplyTypeListFromReturnType
+      val tp = unapplyUnwrap(tp1)
+      val   B = BooleanClass; 
+      val   O = OptionClass; val S = SomeClass; tp.symbol match { // unapplySeqResultToMethodSig
+	case  B                      => Nil
+	case  O                  | S =>
+	  val prod = tp.typeArgs.head
+          getProductArgs(prod)  match {
+            case Some(all @ (x1::x2::xs)) => all       // n >= 2
+            case _                        => prod::Nil // special n == 0 ||  n == 1 
+          }
+	case _ => throw new IllegalArgumentException(tp.symbol + " in not in {boolean, option, some}")
       }
     }
 
-    def unapplySeqResultToMethodSig(tp: Type) = {
-      val ts = optionOfProductElems(tp)
-      val last1 = ts.last.baseType(SeqClass) match {
-        case TypeRef(pre, seqClass, args) => typeRef(pre, RepeatedParamClass, args)
+    /** let type be the result type of the (possibly polymorphic) unapply method
+     *  for type Option[T] or Some[T] 
+     *  -returns T0...Tn-1,Tn* if n>0 and T <: Product[T0...Tn-1,Seq[Tn]]], 
+     *  -returns R* if T = Seq[R]
+     */
+    def unapplyTypeListFromReturnTypeSeq(tp1: Type): List[Type] = {
+      val tp = unapplyUnwrap(tp1)
+      val   O = OptionClass; val S = SomeClass; tp.symbol match { 
+      case  O                  | S =>
+	val ts = unapplyTypeListFromReturnType(tp1)
+	val last1 = ts.last.baseType(SeqClass) match {
+          case TypeRef(pre, seqClass, args) => typeRef(pre, RepeatedParamClass, args)
+	  case _ => throw new IllegalArgumentException("last not seq")
+	}
+	ts.init ::: List(last1)
+	case _ => throw new IllegalArgumentException(tp.symbol + " in not in {option, some}")
       }
-      ts.init ::: List(last1)
+    }
+
+    /** returns type of the unapply method returning T_0...T_n
+     *  for n == 0, boolean
+     *  for n == 1, Some[T0]
+     *  else Some[Product[Ti]]
+    def unapplyReturnType(elems: List[Type], useWildCards: Boolean) = 
+      if (elems.isEmpty)
+	BooleanClass.tpe
+      else if (elems.length == 1)
+	optionType(if(useWildCards) WildcardType else elems(0))
+      else 
+	productType({val es = elems; if(useWildCards) elems map { x => WildcardType} else elems})
+     */
+
+    def unapplyReturnTypeExpected(argsLength: int) = argsLength match {
+      case 0 => BooleanClass.tpe
+      case 1 => optionType(WildcardType)
+      case n => optionType(productType(List.range(0,n).map (arg => WildcardType)))
     }
 
     /** returns unapply or unapplySeq if available */
@@ -193,6 +277,32 @@ trait Definitions requires SymbolTable {
         case _ =>
           false
       }
+
+    def isCorrespondingDelegate(delegateType: Type, functionType: Type): boolean = {
+      var isCD: Boolean = false;
+      if (DelegateClass != null && delegateType != null &&
+	  isSubType(delegateType, DelegateClass.tpe))
+	{
+	  val meth: Symbol = delegateType.member(nme.apply)
+	  meth.tpe match {
+	    case MethodType(delegateParams, delegateReturn) =>
+	      val delegateParamsO = delegateParams.map(pt => {if (pt == definitions.AnyClass.tpe) definitions.ObjectClass.tpe else pt});
+	      if(isFunctionType(functionType))
+		functionType match {
+		  case TypeRef(_, _, args) =>
+		    if(delegateParamsO == args.dropRight(1) &&
+		       delegateReturn == args.last)
+		      isCD = true;
+
+		  case _ => ();
+		}
+
+	    case _ => ()
+	  }
+	}
+      isCD
+    }
+
 /*    val RemoteFunctionClass: Array[Symbol] = new Array(MaxFunctionArity + 1)
       def remoteFunctionApply(n: Int) = getMember(RemoteFunctionClass(n), nme.apply)
       def remoteFunctionType(formals: List[Type], restpe: Type) =
@@ -257,6 +367,7 @@ trait Definitions requires SymbolTable {
 
     // special attributes
     var SerializableAttr: Symbol = _
+    var DeprecatedAttr: Symbol = _
     var BeanPropertyAttr: Symbol = _
     var AnnotationDefaultAttr: Symbol = _
 
@@ -337,7 +448,7 @@ trait Definitions requires SymbolTable {
 
     private def newTypeParam(owner: Symbol, index: int): Symbol =
       owner.newTypeParameter(NoPos, "T" + index)
-        .setInfo(TypeBounds(AllClass.typeConstructor, AnyClass.typeConstructor))
+        .setInfo(mkTypeBounds(AllClass.typeConstructor, AnyClass.typeConstructor))
 
     val boxedClass = new HashMap[Symbol, Symbol]
     val unboxMethod = new HashMap[Symbol, Symbol] // Type -> Method
@@ -351,10 +462,20 @@ trait Definitions requires SymbolTable {
 
     private def newValueClass(name: Name, tag: char): Symbol = {
       def boxedName: String =
-        "scala.runtime.Boxed" + name
+        if (!forMSIL) "scala.runtime.Boxed" + name
+        else "System." + (name match {
+          case nme.Boolean => "Boolean"
+          case nme.Byte => "Byte"
+          case nme.Char => "Char"
+          case nme.Short => "Int16"
+          case nme.Int => "Int32"
+          case nme.Long => "Int64"
+          case nme.Float => "Single"
+          case nme.Double => "Double"
+        })
       val clazz =
         newClass(ScalaPackageClass, name, List(AnyValClass.typeConstructor))
-        .setFlag(ABSTRACT | SEALED)
+        .setFlag(ABSTRACT /* SEALED */) // bq: SEALED is interesting for case class descendants, only
       boxedClass(clazz) = getClass(boxedName)
       boxedArrayClass(clazz) = getClass("scala.runtime.Boxed" + name + "Array")
       refClass(clazz) = getClass("scala.runtime." + name + "Ref")
@@ -399,7 +520,7 @@ trait Definitions requires SymbolTable {
       val stringtype = StringClass.typeConstructor
 
       // init scala.Boolean
-      newParameterlessMethod(BooleanClass, nme.ZNOT, booltype)
+      newParameterlessMethod(BooleanClass, nme.UNARY_!, booltype)
       newMethod(BooleanClass, nme.EQ,   boolparam, booltype)
       newMethod(BooleanClass, nme.NE,   boolparam, booltype)
       newMethod(BooleanClass, nme.ZOR,  boolparam, booltype)
@@ -463,10 +584,12 @@ trait Definitions requires SymbolTable {
         }
 
         // unary operations
-        newParameterlessMethod(clazz, nme.ADD, restype)
-        newParameterlessMethod(clazz, nme.SUB, restype)
-        if (isCardinal)
-          newParameterlessMethod(clazz, nme.NOT, restype)
+        newParameterlessMethod(clazz, nme.UNARY_+, restype)
+        newParameterlessMethod(clazz, nme.UNARY_-, restype)
+
+        if (isCardinal) {
+          newParameterlessMethod(clazz, nme.UNARY_~, restype)
+        }
 
         // binary operations
         val restype2 = if (isCardinal) longtype else restype
@@ -491,6 +614,34 @@ trait Definitions requires SymbolTable {
         initValueClass(FloatClass,  false)
         initValueClass(DoubleClass, false)
       }
+      def addModuleMethod(clazz: Symbol, name: Name, value: Any): Unit = {
+        val owner = clazz.linkedClassOfClass
+        newParameterlessMethod(owner, name, mkConstantType(Constant(value)))
+      }
+      addModuleMethod(ByteClass,  "MinValue",  java.lang.Byte.MIN_VALUE)
+      addModuleMethod(ByteClass,  "MaxValue",  java.lang.Byte.MAX_VALUE)
+      addModuleMethod(ShortClass, "MinValue",  java.lang.Short.MIN_VALUE)
+      addModuleMethod(ShortClass, "MaxValue",  java.lang.Short.MAX_VALUE)
+      addModuleMethod(CharClass,  "MinValue",  java.lang.Character.MIN_VALUE)
+      addModuleMethod(CharClass,  "MaxValue",  java.lang.Character.MAX_VALUE)
+      addModuleMethod(IntClass,   "MinValue",  java.lang.Integer.MIN_VALUE)
+      addModuleMethod(IntClass,   "MaxValue",  java.lang.Integer.MAX_VALUE)
+      addModuleMethod(LongClass,  "MinValue",  java.lang.Long.MIN_VALUE)
+      addModuleMethod(LongClass,  "MaxValue",  java.lang.Long.MAX_VALUE)
+
+      addModuleMethod(FloatClass, "MinValue", -java.lang.Float.MAX_VALUE)
+      addModuleMethod(FloatClass, "MaxValue",  java.lang.Float.MAX_VALUE)
+      addModuleMethod(FloatClass, "Epsilon",   java.lang.Float.MIN_VALUE)
+      addModuleMethod(FloatClass, "NaN",       java.lang.Float.NaN)
+      addModuleMethod(FloatClass, "PositiveInfinity", java.lang.Float.POSITIVE_INFINITY)
+      addModuleMethod(FloatClass, "NegativeInfinity", java.lang.Float.NEGATIVE_INFINITY)
+
+      addModuleMethod(DoubleClass, "MinValue", -java.lang.Double.MAX_VALUE)
+      addModuleMethod(DoubleClass, "MaxValue",  java.lang.Double.MAX_VALUE)
+      addModuleMethod(DoubleClass, "Epsilon",   java.lang.Double.MIN_VALUE)
+      addModuleMethod(DoubleClass, "NaN",       java.lang.Double.NaN)
+      addModuleMethod(DoubleClass, "PositiveInfinity", java.lang.Double.POSITIVE_INFINITY)
+      addModuleMethod(DoubleClass, "NegativeInfinity", java.lang.Double.NEGATIVE_INFINITY)
     }
 
     /** Is symbol a value class? */
@@ -501,9 +652,12 @@ trait Definitions requires SymbolTable {
     def isNumericValueClass(sym: Symbol): boolean =
       (sym ne BooleanClass) && (boxedClass contains sym)
 
+    def isValueType(sym: Symbol) =
+      isValueClass(sym) || unboxMethod.contains(sym)
+
     /** Is symbol a value or array class? */
     def isUnboxedClass(sym: Symbol): boolean =
-      isValueClass(sym) || sym == ArrayClass
+      isValueType(sym) || sym == ArrayClass
 
     def signature(tp: Type): String = {
       def erasure(tp: Type): Type = tp match {
@@ -545,7 +699,7 @@ trait Definitions requires SymbolTable {
       RootClass.info.decls.enter(EmptyPackage)
       RootClass.info.decls.enter(RootPackage)
 
-      JavaLangPackage = getModule("java.lang")
+      JavaLangPackage = getModule(if (forMSIL) "System" else "java.lang")
       ScalaPackage = getModule("scala")
       assert(ScalaPackage ne null, "Scala package is null")
       ScalaPackageClass = ScalaPackage.tpe.symbol
@@ -557,7 +711,7 @@ trait Definitions requires SymbolTable {
       AnyValClass = newClass(ScalaPackageClass, nme.AnyVal, anyparam)
         .setFlag(FINAL | SEALED)
 
-      ObjectClass = getClass("java.lang.Object")
+      ObjectClass = getClass(if (forMSIL) "System.Object" else "java.lang.Object")
 
       AnyRefClass =
         newAlias(ScalaPackageClass, nme.AnyRef, ObjectClass.typeConstructor)
@@ -570,13 +724,17 @@ trait Definitions requires SymbolTable {
       AllClass = newClass(ScalaPackageClass, nme.Nothing, anyparam)
         .setFlag(ABSTRACT | TRAIT | FINAL)
 
-      StringClass = getClass("java.lang.String")
+      StringClass = getClass(if (forMSIL) "System.String" else "java.lang.String")
 
-      ClassClass = getClass("java.lang.Class")
-      ThrowableClass = getClass("java.lang.Throwable")
-      NullPointerExceptionClass = getClass("java.lang.NullPointerException")
+      ClassClass = getClass(if (forMSIL) "System.Type" else "java.lang.Class")
+      ThrowableClass = getClass(if (forMSIL) "System.Exception" else "java.lang.Throwable")
+      NullPointerExceptionClass = getClass(if (forMSIL) "System.NullReferenceException"
+                                           else "java.lang.NullPointerException")
       NonLocalReturnExceptionClass = getClass("scala.runtime.NonLocalReturnException")
 
+      ValueTypeClass = if (forMSIL) getClass("System.ValueType") else null
+      DelegateClass = if (forMSIL) getClass("System.MulticastDelegate") else null
+
       UnitClass =
         newClass(ScalaPackageClass, nme.Unit, List(AnyValClass.typeConstructor))
       abbrvTag(UnitClass) = 'V'
@@ -594,10 +752,12 @@ trait Definitions requires SymbolTable {
 
       // the scala reference classes
       ScalaObjectClass = getClass("scala.ScalaObject")
-      AttributeClass = getClass("scala.Attribute")
+      AnnotationClass = getClass("scala.Annotation")
+      ClassfileAnnotationClass = getClass("scala.ClassfileAnnotation")
+      StaticAnnotationClass = getClass("scala.StaticAnnotation")
       //ChannelClass = getClass("scala.distributed.Channel")
       //RemoteRefClass = getClass("scala.distributed.RemoteRef")
-      if (!forCLDC) {
+      if (!forCLDC && ! forMSIL) {
         CodeClass = getClass("scala.reflect.Code")
         CodeModule = getModule("scala.reflect.Code")
       }
@@ -609,12 +769,14 @@ trait Definitions requires SymbolTable {
       ListClass = getClass("scala.List")
       ListModule = getModule("scala.List")
       ArrayClass = getClass("scala.Array")
-      SerializableClass = if (forCLDC) null else getClass("java.io.Serializable")
+      SerializableClass = if (forMSIL || forCLDC) null else getClass("java.io.Serializable")
       PredefModule = getModule("scala.Predef")
       ConsoleModule = getModule("scala.Console")
       MatchErrorClass = getClass("scala.MatchError")
       MatchErrorModule = getModule("scala.MatchError")
-      IndexOutOfBoundsExceptionClass = getClass("java.lang.IndexOutOfBoundsException")
+      IndexOutOfBoundsExceptionClass =
+        getClass(if (forMSIL) "System.IndexOutOfRangeException"
+                 else "java.lang.IndexOutOfBoundsException")
       //RemoteExecutionModule = getModule("scala.distributed.RemoteExecution")
       ScalaRunTimeModule = getModule("scala.runtime.ScalaRunTime")
       RepeatedParamClass = newCovariantPolyClass(
@@ -623,6 +785,7 @@ trait Definitions requires SymbolTable {
       ByNameParamClass = newCovariantPolyClass(
         ScalaPackageClass, nme.BYNAME_PARAM_CLASS_NAME, tparam => AnyClass.typeConstructor)
       /* <unapply> */
+      UnsealedClass = getClass("scala.unsealed")
       OptionClass = getClass("scala.Option")
 
       for (val i <- 1 to MaxTupleArity) {
@@ -685,11 +848,85 @@ trait Definitions requires SymbolTable {
       BoxedUnitModule = getModule("scala.runtime.BoxedUnit")
       ObjectRefClass = getClass("scala.runtime.ObjectRef")
 
+      if (forMSIL) {
+        val intType = IntClass.typeConstructor;
+        val intParam = List(intType);
+        val longType = LongClass.typeConstructor;
+        val charType = CharClass.typeConstructor;
+        val unitType = UnitClass.typeConstructor;
+        val stringType = StringClass.typeConstructor;
+        val stringParam = List(stringType);
+
+        // additional methods of Object
+        newMethod(ObjectClass, "clone", List(), AnyRefClass.typeConstructor);
+        newMethod(ObjectClass, "wait", List(), unitType);
+        newMethod(ObjectClass, "wait", List(longType), unitType);
+        newMethod(ObjectClass, "wait", List(longType, intType), unitType);
+        newMethod(ObjectClass, "notify", List(), unitType);
+        newMethod(ObjectClass, "notifyAll", List(), unitType);
+
+        // additional methods of String
+        newMethod(StringClass, "length", List(), intType);
+        newMethod(StringClass, "compareTo", stringParam, intType);
+        newMethod(StringClass, "charAt", intParam, charType);
+        newMethod(StringClass, "concat", stringParam, stringType);
+        newMethod(StringClass, "indexOf", intParam, intType);
+        newMethod(StringClass, "indexOf", List(intType, intType), intType);
+        newMethod(StringClass, "indexOf", stringParam, intType);
+        newMethod(StringClass, "indexOf", List(stringType, intType), intType);
+        newMethod(StringClass, "lastIndexOf", intParam, intType);
+        newMethod(StringClass, "lastIndexOf", List(intType, intType), intType);
+        newMethod(StringClass, "lastIndexOf", stringParam, intType);
+        newMethod(StringClass, "lastIndexOf", List(stringType, intType), intType);
+        newMethod(StringClass, "toLowerCase", List(), stringType);
+        newMethod(StringClass, "toUpperCase", List(), stringType);
+        newMethod(StringClass, "startsWith", stringParam, booltype);
+        newMethod(StringClass, "endsWith", stringParam, booltype);
+        newMethod(StringClass, "substring", intParam, stringType);
+        newMethod(StringClass, "substring", List(intType, intType), stringType);
+        newMethod(StringClass, "trim", List(), stringType);
+        newMethod(StringClass, "intern", List(), stringType);
+        newMethod(StringClass, "replace", List(charType, charType), stringType);
+        newMethod(StringClass, "toCharArray", List(),
+                  appliedType(ArrayClass.typeConstructor, List(charType)));
+
+	// Delegate_scalaCallerInfos = new HashMap()
+	Delegate_scalaCallerTargets = new HashMap()
+      }
+
       AnnotationDefaultAttr = newClass(RootClass,
                                        nme.AnnotationDefaultATTR,
-                                       List(AttributeClass.typeConstructor))
+                                       List(AnnotationClass.typeConstructor))
       SerializableAttr = getClass("scala.serializable")
-      BeanPropertyAttr = if (forCLDC) null else getClass("scala.reflect.BeanProperty")
+      BeanPropertyAttr = if (forCLDC || forMSIL) null else getClass("scala.reflect.BeanProperty")
+      DeprecatedAttr = getClass("scala.deprecated")
+    }
+
+    var nbScalaCallers: Int = 0
+    def newScalaCaller(delegateType: Type): Symbol = {
+      assert(forMSIL, "scalaCallers can only be created if target is .NET")
+      // object: reference to object on which to call (scala-)metod
+      val paramTypes: List[Type] = List(ObjectClass.tpe)
+      val name: String =  "$scalaCaller$$" + nbScalaCallers
+      // tparam => resultType, which is the resultType of PolyType, i.e. the result type after applying the
+      // type parameter =-> a MethodType in this case
+      // TODO: set type bounds manually (-> MulticastDelegate), see newTypeParam
+      val newCaller = newMethod(DelegateClass, name, paramTypes, delegateType) setFlag (FINAL | STATIC)
+      // val newCaller = newPolyMethod(DelegateClass, name,
+      // tparam => MethodType(paramTypes, tparam.typeConstructor)) setFlag (FINAL | STATIC)
+      Delegate_scalaCallers = Delegate_scalaCallers ::: List(newCaller)
+      nbScalaCallers = nbScalaCallers + 1
+      newCaller
+    }
+
+    // def addScalaCallerInfo(scalaCaller: Symbol, methSym: Symbol, delType: Type) = {
+    // assert(Delegate_scalaCallers contains scalaCaller)
+    // Delegate_scalaCallerInfos += scalaCaller -> Pair(methSym, delType)
+    // }
+
+    def addScalaCallerInfo(scalaCaller: Symbol, methSym: Symbol) = {
+      assert(Delegate_scalaCallers contains scalaCaller)
+      Delegate_scalaCallerTargets += scalaCaller -> methSym
     }
   }
 }
