@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Symbols.scala 9678 2007-01-23 13:46:50Z spoon $
+// $Id: Symbols.scala 10036 2007-02-19 13:56:55Z odersky $
 
 package scala.tools.nsc.symtab
 
@@ -20,9 +20,8 @@ trait Symbols requires SymbolTable {
   var typeSymbolCount = 0
   var classSymbolCount = 0
 
-  type AttrInfo = Triple[Type, List[Constant], List[Pair[Name,Constant]]]
-
   val emptySymbolArray = new Array[Symbol](0)
+  val emptySymbolSet = Set.empty[Symbol]
   type PositionType;
   def NoPos : PositionType;
   def FirstPos : PositionType;
@@ -152,7 +151,7 @@ trait Symbols requires SymbolTable {
     final def newAnonymousFunctionClass(pos: PositionType) = {
       val anonfun = newClass(pos, nme.ANON_FUN_NAME.toTypeName)
       anonfun.attributes =
-        Triple(definitions.SerializableAttr.tpe, List(), List()) :: anonfun.attributes
+        AttrInfo(definitions.SerializableAttr.tpe, List(), List()) :: anonfun.attributes
       anonfun
     }
     final def newRefinementClass(pos: PositionType) =
@@ -190,7 +189,7 @@ trait Symbols requires SymbolTable {
     final def isModule = isTerm && hasFlag(MODULE)
     final def isStaticModule = isModule && isStatic && !isMethod
     final def isPackage = isModule && hasFlag(PACKAGE)
-    final def isThisSym = isTerm && name == nme.this_
+    final def isThisSym = isTerm && owner.thisSym == this
     final def isThisSkolem = isTerm && deSkolemize != this
     final def isError = hasFlag(IS_ERROR)
     final def isErroneous = isError || isInitialized && tpe.isErroneous
@@ -210,7 +209,7 @@ trait Symbols requires SymbolTable {
     final def isEmptyPackage = isPackage && name == nme.EMPTY_PACKAGE_NAME
     final def isEmptyPackageClass = isPackageClass && name == nme.EMPTY_PACKAGE_NAME.toTypeName
     def isDeprecated = 
-      attributes exists (attr => attr._1.symbol == DeprecatedAttr)
+      attributes exists (attr => attr.atp.symbol == DeprecatedAttr)
 
     /** Does this symbol denote a wrapper object of the interpreter or its class? */
     final def isInterpreterWrapper = 
@@ -223,6 +222,10 @@ trait Symbols requires SymbolTable {
     final def isStable =
       isTerm && !hasFlag(MUTABLE) && (!hasFlag(METHOD | BYNAMEPARAM) || hasFlag(STABLE))
 
+    /** Is this symbol a private local */
+    final def isPrivateLocal = 
+      hasFlag(PRIVATE) && hasFlag(LOCAL)
+
     /** Does this symbol denote the primary constructor of its enclosing class? */
     final def isPrimaryConstructor =
       isConstructor && owner.primaryConstructor == this
@@ -530,7 +533,7 @@ trait Symbols requires SymbolTable {
     }
 
     def getAttributes(clazz: Symbol): List[AttrInfo] =
-      attributes.filter(._1.symbol.isNonBottomSubClass(clazz))
+      attributes.filter(.atp.symbol.isNonBottomSubClass(clazz))
 
     /** Reset symbol to initial state
      */
@@ -667,6 +670,7 @@ trait Symbols requires SymbolTable {
       owner.info.decl(nme.getterToLocal(if (isSetter) nme.setterToGetter(name) else name))
     }
 
+    /** The implementation class of a trait */
     final def implClass: Symbol = owner.info.decl(nme.implClassName(name))
 
     /** The class that is logically an outer class of given `clazz'.
@@ -817,9 +821,11 @@ trait Symbols requires SymbolTable {
     final def setter(base: Symbol): Symbol =
       base.info.decl(nme.getterToSetter(nme.getterName(name))) filter (.hasFlag(ACCESSOR))
 
-    /** The case factory corresponding to this case class */
-    final def caseFactory: Symbol = 
-      owner.info.decl(name.toTermName).suchThat(.isCaseFactory)
+    /** The case factory corresponding to this case class
+     *  @pre case class is a member of some other class or package
+     */
+    final def caseFactory: Symbol =
+      initialize.owner.info.decl(name.toTermName).suchThat(.isCaseFactory)
 
     /** If this symbol is a skolem, its corresponding type parameter, otherwise this */
     def deSkolemize: Symbol = this
@@ -867,6 +873,14 @@ trait Symbols requires SymbolTable {
     def isFromClassFile: Boolean =
       (if (isModule) moduleClass else toplevelClass).isFromClassFile
 
+    /** If this is a sealed class, its known direct subclasses. Otherwise Set.empty */
+    def children: Set[Symbol] = emptySymbolSet
+
+    /** Declare given subclass `sym' of this sealed class */
+    def addChild(sym: Symbol) {
+      throw new Error("addChild inapplicable for " + this)
+    }
+      
 
 // ToString -------------------------------------------------------------------
 
@@ -1175,6 +1189,7 @@ trait Symbols requires SymbolTable {
 
     /** The classfile from which this class was loaded. Maybe null. */
     var classFile: AbstractFile = null;
+    
     private var source: AbstractFile = null
     override def sourceFile =
       if (owner.isPackageClass) source else super.sourceFile
@@ -1217,7 +1232,7 @@ trait Symbols requires SymbolTable {
       val period = thisTypePeriod
       if (period != currentPeriod) {
         thisTypePeriod = currentPeriod
-        if (!isValid(period)) thisTypeCache = ThisType(this)
+        if (!isValid(period)) thisTypeCache = mkThisType(this)
       }
       thisTypeCache
     }
@@ -1237,13 +1252,20 @@ trait Symbols requires SymbolTable {
     override def cloneSymbolImpl(owner: Symbol): Symbol = {
       assert(!isModuleClass)
       val clone = new ClassSymbol(owner, pos, name)
-      if (thisSym != this) clone.typeOfThis = typeOfThis
+      if (thisSym != this) {
+        clone.typeOfThis = typeOfThis
+        clone.thisSym.name = thisSym.name
+      }
       clone
     }
 
     override def sourceModule =
       if (isModuleClass) linkedModuleOfClass else NoSymbol
 
+    private var childSet: Set[Symbol] = emptySymbolSet
+    override def children: Set[Symbol] = childSet
+    override def addChild(sym: Symbol) { childSet = childSet + sym }
+
     if (util.Statistics.enabled) classSymbolCount = classSymbolCount + 1
   }
 
@@ -1287,6 +1309,13 @@ trait Symbols requires SymbolTable {
     def cloneSymbolImpl(owner: Symbol): Symbol = throw new Error()
   }
 
+  case class AttrInfo(atp: Type, args: List[Constant], assocs: List[Pair[Name, Constant]]) {
+    override def toString: String =
+      atp +
+      (if (args.isEmpty) "" else args.mkString("(", ", ", ")")) +
+      (assocs map { case Pair(x, y) => x+" = "+y } mkString ("{", ", ", "}"))
+  }
+
   def cloneSymbols(syms: List[Symbol]): List[Symbol] = {
     val syms1 = syms map (.cloneSymbol)
     for (val sym1 <- syms1) sym1.setInfo(sym1.info.substSym(syms, syms1))
