@@ -2,13 +2,13 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: Pickler.scala 9685 2007-01-23 15:34:40Z spoon $
+// $Id: Pickler.scala 10036 2007-02-19 13:56:55Z odersky $
 
 package scala.tools.nsc.symtab.classfile
 
 import java.lang.{Float, Double}
 import scala.collection.mutable.HashMap
-import scala.tools.nsc.util.Position
+import scala.tools.nsc.util.{Position, ShowPickled}
 import Flags._
 import PickleFormat._
 
@@ -105,11 +105,16 @@ abstract class Pickler extends SubComponent {
         putSymbol(sym.owner)
         putSymbol(sym.privateWithin)
         putType(sym.info)
-        if (sym.thisSym != sym)
+        if (sym.thisSym.tpe != sym.tpe)
           putType(sym.typeOfThis);
         putSymbol(sym.alias)
+        if (!sym.children.isEmpty) 
+          putChildren(sym, sym.children.toList.sort((x, y) => x isLess y))
 
-        //for (val attr <- sym.attributes) putAttribute(sym, attr);
+        for (val attr <- sym.attributes.reverse) {
+          if (attr.atp.symbol isNonBottomSubClass definitions.StaticAnnotationClass)
+            putAnnotation(sym, attr)
+        }
       } else if (sym != NoSymbol) {
         putEntry(if (sym.isModuleClass) sym.name.toTermName else sym.name)
         if (!sym.owner.isRoot) putSymbol(sym.owner)
@@ -162,12 +167,19 @@ abstract class Pickler extends SubComponent {
         if (c.tag == StringTag) putEntry(newTermName(c.stringValue))
         else if (c.tag == ClassTag) putEntry(c.typeValue)
       }
-/*
-    private def putAttribute(attr: AttrInfo): unit = if (putEntry(attr)) {
-      putType(attr._1);
-      for (val c <- attr._2) putConstant(c);
+
+    private def putChildren(sym: Symbol, children: List[Symbol]): unit = {
+      assert(putEntry(Pair(sym, children)))
+      children foreach putSymbol
     }
-*/
+
+    private def putAnnotation(sym: Symbol, attr: AttrInfo): unit = {
+      assert(putEntry(Pair(sym, attr)))
+      putType(attr.atp)
+      for (val c <- attr.args) putConstant(c)
+      for (val Pair(name, c) <- attr.assocs) { putEntry(name); putConstant(c) }
+    }
+
     // Phase 2 methods: Write all entries to byte array ------------------------------
 
     private val buf = new PickleBuffer(new Array[byte](4096), -1, 0)
@@ -224,7 +236,7 @@ abstract class Pickler extends SubComponent {
           tag
         case sym: ClassSymbol =>
           val posOffset = writeSymInfo(sym)
-          if (sym.thisSym != sym) writeRef(sym.typeOfThis)
+          if (sym.thisSym.tpe != sym.tpe) writeRef(sym.typeOfThis)
           CLASSsym + posOffset
         case sym: TypeSymbol =>
           val posOffset = writeSymInfo(sym)
@@ -265,14 +277,18 @@ abstract class Pickler extends SubComponent {
           else if (c.tag == StringTag) writeRef(newTermName(c.stringValue))
           else if (c.tag == ClassTag) writeRef(c.typeValue)
           LITERAL + c.tag
-/*
-        case Pair(tp, cs) =>
-          writeRef(tp); 
-          for (val c <- cs) writeRef(cs);
+        case AttributedType(attribs, tp) => 
+          writeBody(tp) // obviously, this should be improved
+        case Pair(target: Symbol, attr @ AttrInfo(atp, args, assocs)) =>
+          writeRef(target)
+          writeRef(atp)
+          for (val c <- args) writeRef(c)
+          for (val Pair(name, c) <- assocs) { writeRef(name); writeRef(c) }
           ATTRIBUTE
-*/
-        case AttributedType(attribs, tp)
-          => writeBody(tp) // obviously, this should be improved
+        case Pair(target: Symbol, children: List[_]) =>
+          writeRef(target)
+          for (val c <- children) writeRef(c.asInstanceOf[Symbol])
+          CHILDREN
         case _ =>
           throw new FatalError("bad entry: " + entry + " " + entry.getClass())//debug
       }
@@ -290,6 +306,10 @@ abstract class Pickler extends SubComponent {
       writeNat(ep)
       if (settings.debug.value) log("" + ep + " entries")//debug
       for (val i <- 0 until ep) writeEntry(entries(i));
+      if (settings.Xshowcls.value == rootName.toString) {
+        readIndex = 0
+        ShowPickled.printFile(this, Console.out)
+      }
     }
 
     override def toString() = "" + rootName + " in " + rootOwner
