@@ -2,7 +2,7 @@
  * Copyright 2005-2006 LAMP/EPFL
  * @author Martin Odersky
  */
-// $Id: Erasure.scala 9460 2006-12-04 17:41:36Z odersky $
+// $Id: Erasure.scala 10036 2007-02-19 13:56:55Z odersky $
 
 package scala.tools.nsc.transform
 
@@ -57,7 +57,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
    *    </li>
    *  </ul>
    */
-  private val erasure = new TypeMap {
+  val erasure = new TypeMap {
     def apply(tp: Type): Type = tp match {
       case ConstantType(_) =>
         tp
@@ -85,7 +85,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
         else apply(parents.head)
       case ClassInfoType(parents, decls, clazz) =>
         ClassInfoType(
-          if ((clazz == ObjectClass) || (isValueClass(clazz))) List() 
+          if ((clazz == ObjectClass) || (isValueType(clazz))) List() 
           else if (clazz == ArrayClass) List(erasedTypeRef(ObjectClass))
           else removeDoubleObject(parents map this),
           decls, clazz)
@@ -138,7 +138,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
     else if (sym == Object_isInstanceOf || sym == ArrayClass) 
       PolyType(sym.info.typeParams, erasure(sym.info.resultType))
     else if (sym.isAbstractType) 
-      TypeBounds(WildcardType, WildcardType)
+      mkTypeBounds(WildcardType, WildcardType)
     else if (sym.isTerm && sym.owner == ArrayClass) {
       if (sym.isClassConstructor)
         tp match {
@@ -308,7 +308,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
           }
         else gen.mkAttributedCast(tree, pt)
       } else gen.mkAttributedCast(tree, pt)
-        
+
     /** Is symbol a member of unboxed arrays (which will be expanded directly
      *  later)?
      *
@@ -330,10 +330,10 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
      */
     private def adaptToType(tree: Tree, pt: Type): Tree = {
       if (settings.debug.value && pt != WildcardType)
-        log("adapting " + tree + ":" + tree.tpe + " to " + pt)//debug
-      if (tree.tpe <:< pt) 
+        log("adapting " + tree + ":" + tree.tpe + " : " +  tree.tpe.parents + " to " + pt)//debug
+      if (tree.tpe <:< pt)
         tree
-      else if (isUnboxedClass(tree.tpe.symbol) && !isUnboxedClass(pt.symbol)) 
+      else if (isUnboxedClass(tree.tpe.symbol) && !isUnboxedClass(pt.symbol))
         adaptToType(box(tree), pt)
       else if (tree.tpe.isInstanceOf[MethodType] && tree.tpe.paramTypes.isEmpty) {
         assert(tree.symbol.isStable);
@@ -403,6 +403,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
      *  </ul>
      */
     private def adaptMember(tree: Tree): Tree = {
+      //Console.println("adaptMember: " + tree);
       tree match {
         case Apply(Select(New(tpt), name), args) if (tpt.tpe.symbol == BoxedArrayClass) =>
           assert(name == nme.CONSTRUCTOR);
@@ -417,7 +418,7 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
           if (isNumericValueClass(qualClass) && isNumericValueClass(targClass))
             // convert numeric type casts
             atPos(tree.pos)(Apply(Select(qual1, "to" + targClass.name), List()))
-          else if (isValueClass(targClass) ||
+          else if (isValueType(targClass) ||
                    (targClass == ArrayClass && (qualClass isNonBottomSubClass BoxedArrayClass)))
             unbox(qual1, targ.tpe)
           else if (targClass == ArrayClass && qualClass == ObjectClass)
@@ -435,10 +436,10 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
             adaptMember(atPos(tree.pos)(Select(qual, getMember(ObjectClass, name))))
           else {
             var qual1 = typedQualifier(qual);
-            if ((isValueClass(qual1.tpe.symbol) && !isUnboxedValueMember(tree.symbol)) || 
+            if ((isValueType(qual1.tpe.symbol) && !isUnboxedValueMember(tree.symbol)) || 
                 (qual1.tpe.symbol == ArrayClass && !isUnboxedArrayMember(tree.symbol)))
               qual1 = box(qual1);
-            else if (!isValueClass(qual1.tpe.symbol) && isUnboxedValueMember(tree.symbol))
+            else if (!isValueType(qual1.tpe.symbol) && isUnboxedValueMember(tree.symbol))
               qual1 = unbox(qual1, tree.symbol.owner.tpe)
             else if (tree.symbol.owner == ArrayClass && qual1.tpe.symbol == ObjectClass)
               qual1 = cast(qual1, BoxedArrayClass.tpe)
@@ -716,10 +717,10 @@ abstract class Erasure extends AddInterfaces with typechecker.Analyzer {
       override def transform(tree: Tree): Tree = {
         if (tree.symbol == ArrayClass) return tree
         val tree1 = tree match {
-          case ClassDef(mods, name, tparams, tpt, impl) =>
+          case ClassDef(mods, name, tparams, self, impl) =>
             if (settings.debug.value)
               log("defs of " + tree.symbol + " = " + tree.symbol.info.decls)
-            copy.ClassDef(tree, mods, name, List(), tpt, impl)
+            copy.ClassDef(tree, mods, name, List(), emptyValDef, impl)
           case DefDef(mods, name, tparams, vparamss, tpt, rhs) =>
             copy.DefDef(tree, mods, name, List(), vparamss, tpt, rhs)
           case AbsTypeDef(_, _, _, _) =>
