@@ -6,10 +6,10 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: volatile.scala 8756 2006-09-22 16:00:23Z michelou $
+// $Id: volatile.scala 9951 2007-02-09 19:09:06Z odersky $
 
 
 package scala
 
 
-class volatile extends Attribute
+class volatile extends Annotation
