@@ -6,10 +6,10 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: MalformedAttributeException.scala 5889 2006-03-05 00:33:02Z mihaylov $
+// $Id: MalformedAttributeException.scala 8744 2006-09-22 09:38:30Z michelou $
 
 
-package scala.xml;
+package scala.xml
 
 
-case class MalformedAttributeException(msg:String) extends java.lang.RuntimeException(msg);
+case class MalformedAttributeException(msg: String) extends java.lang.RuntimeException(msg)
