@@ -6,7 +6,7 @@
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: MetaData.scala 9546 2007-01-03 15:56:13Z odersky $
+// $Id: MetaData.scala 9886 2007-02-05 16:18:55Z odersky $
 
 
 package scala.xml
@@ -102,7 +102,7 @@ abstract class MetaData extends Iterable[MetaData] {
     }
 
   /** returns an iterator on attributes */
-  def elements = new Iterator[MetaData] {
+  def elements: Iterator[MetaData] = new Iterator[MetaData] {
     var x: MetaData = MetaData.this
     def hasNext = Null != x
     def next = {
